@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1994, 1995, 1996, 1997, 1999, 2002, 2003, 2004, 2005,
-              2007 John W. Eaton
+              2007, 2008 John W. Eaton
 
 This file is part of Octave.
 
@@ -34,7 +34,7 @@ along with Octave; see the file COPYING.  If not, see
 #include <base-lu.h>
 #include <base-lu.cc>
 
-template class base_lu <ComplexMatrix, Complex, Matrix, double>;
+template class base_lu <ComplexMatrix>;
 
 // Define the constructor for this particular derivation.
 
@@ -61,10 +61,7 @@ ComplexLU::ComplexLU (const ComplexMatrix& a)
 
   F77_XFCN (zgetrf, ZGETRF, (a_nr, a_nc, tmp_data, a_nr, pipvt, info));
 
-  if (f77_exception_encountered)
-    (*current_liboctave_error_handler) ("unrecoverable error in zgetrf");
-  else
-    ipvt -= static_cast<octave_idx_type> (1);
+  ipvt -= static_cast<octave_idx_type> (1);
 }
 
 /*
