@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2004, 2005, 2007 David Bateman
+Copyright (C) 2004, 2005, 2007, 2008 David Bateman
 Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004 Andy Adler
 
 This file is part of Octave.
@@ -112,6 +112,17 @@ public:
     { return Sparse<T>::ipermute (vec); }
 
 
+  MSparse<T> diag (octave_idx_type k = 0) const
+  {
+    return Sparse<T>::diag (k);
+  }
+
+ template <class U, class F>
+  MSparse<U> map (F fcn) const
+  {
+    return Sparse<T>::template map<U> (fcn);
+  }
+
   // Currently, the OPS functions don't need to be friends, but that
   // may change.
 
