@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2004, 2005, 2006, 2007 David Bateman
+Copyright (C) 2004, 2005, 2006, 2007, 2008 David Bateman
 Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004 Andy Adler
 
 This file is part of Octave.
@@ -28,6 +28,7 @@ along with Octave; see the file COPYING.  If not, see
 #include <vector>
 
 #include "lo-error.h"
+#include "oct-locbuf.h"
 
 #include "SparseCmplxLU.h"
 #include "oct-spparms.h"
@@ -42,7 +43,7 @@ template class sparse_base_lu <SparseComplexMatrix, Complex, SparseMatrix, doubl
 #include "oct-sparse.h"
 
 SparseComplexLU::SparseComplexLU (const SparseComplexMatrix& a, 
-				  double piv_thres)
+				  const Matrix& piv_thres, bool scale)
 {
 #ifdef HAVE_UMFPACK
   octave_idx_type nr = a.rows ();
@@ -56,20 +57,24 @@ SparseComplexLU::SparseComplexLU (const SparseComplexMatrix& a,
   double tmp = octave_sparse_params::get_key ("spumoni");
   if (!xisnan (tmp))
     Control (UMFPACK_PRL) = tmp;
-  if (piv_thres >= 0.)
+  if (piv_thres.nelem() == 2)
     {
-      piv_thres = (piv_thres > 1. ? 1. : piv_thres);
-      Control (UMFPACK_SYM_PIVOT_TOLERANCE) = piv_thres;
-      Control (UMFPACK_PIVOT_TOLERANCE) = piv_thres;
+      tmp = (piv_thres (0) > 1. ? 1. : piv_thres (0));
+      if (!xisnan (tmp))
+	Control (UMFPACK_PIVOT_TOLERANCE) = tmp;
+      tmp = (piv_thres (1) > 1. ? 1. : piv_thres (1));
+      if (!xisnan (tmp))
+	Control (UMFPACK_SYM_PIVOT_TOLERANCE) = tmp;
     }
   else
     {
       tmp = octave_sparse_params::get_key ("piv_tol");
       if (!xisnan (tmp))
-	{
+	Control (UMFPACK_PIVOT_TOLERANCE) = tmp;
+
+      tmp = octave_sparse_params::get_key ("sym_tol");
+      if (!xisnan (tmp))
 	  Control (UMFPACK_SYM_PIVOT_TOLERANCE) = tmp;
-      Control (UMFPACK_PIVOT_TOLERANCE) = tmp;
-	}
     }
 
   // Set whether we are allowed to modify Q or not
@@ -78,7 +83,10 @@ SparseComplexLU::SparseComplexLU (const SparseComplexMatrix& a,
     Control (UMFPACK_FIXQ) = tmp;
 
   // Turn-off UMFPACK scaling for LU 
-  Control (UMFPACK_SCALE) = UMFPACK_SCALE_NONE;
+  if (scale)
+    Control (UMFPACK_SCALE) = UMFPACK_SCALE_SUM;
+  else
+    Control (UMFPACK_SCALE) = UMFPACK_SCALE_NONE;
 
   UMFPACK_ZNAME (report_control) (control);
 
@@ -88,14 +96,14 @@ SparseComplexLU::SparseComplexLU (const SparseComplexMatrix& a,
 
   UMFPACK_ZNAME (report_matrix) (nr, nc, Ap, Ai,
 				 reinterpret_cast<const double *> (Ax),
-				 NULL, 1, control);
+				 0, 1, control);
 
   void *Symbolic;
   Matrix Info (1, UMFPACK_INFO);
   double *info = Info.fortran_vec ();
   int status = UMFPACK_ZNAME (qsymbolic) (nr, nc, Ap, Ai, 
 					  reinterpret_cast<const double *> (Ax),
-					  NULL, NULL,
+					  0, 0,
 					  &Symbolic, control, info);
 
   if (status < 0)
@@ -115,7 +123,7 @@ SparseComplexLU::SparseComplexLU (const SparseComplexMatrix& a,
       void *Numeric;
       status = UMFPACK_ZNAME (numeric) (Ap, Ai,
 					reinterpret_cast<const double *> (Ax),
-					NULL, Symbolic, &Numeric, control,
+					0, Symbolic, &Numeric, control,
 					info);
       UMFPACK_ZNAME (free_symbolic) (&Symbolic) ;
 
@@ -173,6 +181,15 @@ SparseComplexLU::SparseComplexLU (const SparseComplexMatrix& a,
 	      octave_idx_type *Uj = Ufact.ridx ();
 	      Complex *Ux = Ufact.data ();
 	      
+	      Rfact = SparseMatrix (nr, nr, nr);
+	      for (octave_idx_type i = 0; i < nr; i++)
+		{
+		  Rfact.xridx (i) = i;
+		  Rfact.xcidx (i) = i;
+		}
+	      Rfact.xcidx (nr) = nr;
+	      double *Rx = Rfact.data ();
+
 	      P.resize (nr);
 	      octave_idx_type *p = P.fortran_vec ();
 
@@ -182,14 +199,14 @@ SparseComplexLU::SparseComplexLU (const SparseComplexMatrix& a,
 	      octave_idx_type do_recip;
 	      status = UMFPACK_ZNAME (get_numeric) (Ltp, Ltj,
 						    reinterpret_cast<double *> (Ltx),
-						    NULL, Up, Uj,
+						    0, Up, Uj,
 						    reinterpret_cast <double *> (Ux),
-						    NULL, p, q, NULL, NULL,
-						    &do_recip, NULL, Numeric);
+						    0, p, q, 0, 0,
+						    &do_recip, Rx, Numeric);
 
 	      UMFPACK_ZNAME (free_numeric) (&Numeric) ;
 
-	      if (status < 0 || do_recip)
+	      if (status < 0)
 		{
 		  (*current_liboctave_error_handler) 
 		    ("SparseComplexLU::SparseComplexLU extracting LU factors failed");
@@ -200,15 +217,19 @@ SparseComplexLU::SparseComplexLU (const SparseComplexMatrix& a,
 		{
 		  Lfact = Lfact.transpose ();
 
+		  if (do_recip)
+		    for (octave_idx_type i = 0; i < nr; i++)
+		      Rx[i] = 1.0 / Rx[i];
+
 		  UMFPACK_ZNAME (report_matrix) (nr, n_inner,
 					    Lfact.cidx (), Lfact.ridx (), 
 					    reinterpret_cast<double *> (Lfact.data()), 
-					    NULL, 1, control);
+					    0, 1, control);
 
 		  UMFPACK_ZNAME (report_matrix) (n_inner, nc,
 					    Ufact.cidx (), Ufact.ridx (), 
 					    reinterpret_cast<double *> (Ufact.data()), 
-					    NULL, 1, control);
+					    0, 1, control);
 		  UMFPACK_ZNAME (report_perm) (nr, p, control);
 		  UMFPACK_ZNAME (report_perm) (nc, q, control);
 		}
@@ -224,8 +245,9 @@ SparseComplexLU::SparseComplexLU (const SparseComplexMatrix& a,
 
 SparseComplexLU::SparseComplexLU (const SparseComplexMatrix& a, 
 				  const ColumnVector& Qinit, 
-				  double piv_thres, bool FixedQ,
-				  double droptol, bool milu, bool udiag)
+				  const Matrix& piv_thres, bool scale,
+				  bool FixedQ, double droptol, 
+				  bool milu, bool udiag)
 {
 #ifdef HAVE_UMFPACK
   if (milu)
@@ -244,20 +266,24 @@ SparseComplexLU::SparseComplexLU (const SparseComplexMatrix& a,
       double tmp = octave_sparse_params::get_key ("spumoni");
       if (!xisnan (tmp))
 	Control (UMFPACK_PRL) = tmp;
-      if (piv_thres >= 0.)
+      if (piv_thres.nelem() == 2)
 	{
-	  piv_thres = (piv_thres > 1. ? 1. : piv_thres);
-	  Control (UMFPACK_SYM_PIVOT_TOLERANCE) = piv_thres;
-	  Control (UMFPACK_PIVOT_TOLERANCE) = piv_thres;
+	  tmp = (piv_thres (0) > 1. ? 1. : piv_thres (0));
+	  if (!xisnan (tmp))
+	    Control (UMFPACK_PIVOT_TOLERANCE) = tmp;
+	  tmp = (piv_thres (1) > 1. ? 1. : piv_thres (1));
+	  if (!xisnan (tmp))
+	    Control (UMFPACK_SYM_PIVOT_TOLERANCE) = tmp;
 	}
       else
 	{
 	  tmp = octave_sparse_params::get_key ("piv_tol");
 	  if (!xisnan (tmp))
-	    {
-	      Control (UMFPACK_SYM_PIVOT_TOLERANCE) = tmp;
-	      Control (UMFPACK_PIVOT_TOLERANCE) = tmp;
-	    }
+	    Control (UMFPACK_PIVOT_TOLERANCE) = tmp;
+
+	  tmp = octave_sparse_params::get_key ("sym_tol");
+	  if (!xisnan (tmp))
+	    Control (UMFPACK_SYM_PIVOT_TOLERANCE) = tmp;
 	}
 
       if (droptol >= 0.)
@@ -274,7 +300,10 @@ SparseComplexLU::SparseComplexLU (const SparseComplexMatrix& a,
 	}
 
       // Turn-off UMFPACK scaling for LU 
-      Control (UMFPACK_SCALE) = UMFPACK_SCALE_NONE;
+      if (scale)
+	Control (UMFPACK_SCALE) = UMFPACK_SCALE_SUM;
+      else
+	Control (UMFPACK_SCALE) = UMFPACK_SCALE_NONE;
 
       UMFPACK_ZNAME (report_control) (control);
 
@@ -283,7 +312,7 @@ SparseComplexLU::SparseComplexLU (const SparseComplexMatrix& a,
       const Complex *Ax = a.data ();
 
       UMFPACK_ZNAME (report_matrix) (nr, nc, Ap, Ai, 
-				reinterpret_cast<const double *> (Ax), NULL,
+				reinterpret_cast<const double *> (Ax), 0,
 				1, control);
 
       void *Symbolic;
@@ -301,7 +330,7 @@ SparseComplexLU::SparseComplexLU (const SparseComplexMatrix& a,
 
 	status = UMFPACK_ZNAME (qsymbolic) (nr, nc, Ap, Ai, 
 				       reinterpret_cast<const double *> (Ax),
-				       NULL, qinit, &Symbolic, control, 
+				       0, qinit, &Symbolic, control, 
 				       info);
       } while (0);
 
@@ -321,7 +350,7 @@ SparseComplexLU::SparseComplexLU (const SparseComplexMatrix& a,
 
 	  void *Numeric;
 	  status = UMFPACK_ZNAME (numeric) (Ap, Ai, 
-				       reinterpret_cast<const double *> (Ax), NULL,
+				       reinterpret_cast<const double *> (Ax), 0,
 				       Symbolic, &Numeric, control, info) ;
 	  UMFPACK_ZNAME (free_symbolic) (&Symbolic) ;
 
@@ -379,6 +408,15 @@ SparseComplexLU::SparseComplexLU (const SparseComplexMatrix& a,
 		  octave_idx_type *Uj = Ufact.ridx ();
 		  Complex *Ux = Ufact.data ();
 	      
+		  Rfact = SparseMatrix (nr, nr, nr);
+		  for (octave_idx_type i = 0; i < nr; i++)
+		    {
+		      Rfact.xridx (i) = i;
+		      Rfact.xcidx (i) = i;
+		    }
+		  Rfact.xcidx (nr) = nr;
+		  double *Rx = Rfact.data ();
+
 		  P.resize (nr);
 		  octave_idx_type *p = P.fortran_vec ();
 
@@ -389,36 +427,39 @@ SparseComplexLU::SparseComplexLU (const SparseComplexMatrix& a,
 		  status = 
 		    UMFPACK_ZNAME (get_numeric) (Ltp, Ltj, 
 					    reinterpret_cast<double *> (Ltx),
-					    NULL, Up, Uj,
+					    0, Up, Uj,
 					    reinterpret_cast<double *> (Ux), 
-					    NULL, p, q, NULL, NULL, 
-					    &do_recip, NULL, Numeric) ;
+					    0, p, q, 0, 0, 
+					    &do_recip, Rx, Numeric) ;
 
 		  UMFPACK_ZNAME (free_numeric) (&Numeric) ;
 
-		  if (status < 0 || do_recip)
+		  if (status < 0)
 		    {
 		      (*current_liboctave_error_handler) 
 			("SparseComplexLU::SparseComplexLU extracting LU factors failed");
 
-		      UMFPACK_ZNAME (report_status) (control, 
-								     status);
+		      UMFPACK_ZNAME (report_status) (control, status);
 		    }
 		  else
 		    {
 		      Lfact = Lfact.transpose ();
 
+		      if (do_recip)
+			for (octave_idx_type i = 0; i < nr; i++)
+			  Rx[i] = 1.0 / Rx[i];
+
 		      UMFPACK_ZNAME (report_matrix) (nr, n_inner, 
 						Lfact.cidx (), 
 						Lfact.ridx (), 
 						reinterpret_cast<double *> (Lfact.data()), 
-						NULL, 1, control);
+						0, 1, control);
 
 		      UMFPACK_ZNAME (report_matrix) (n_inner, nc, 
 						Ufact.cidx (), 
 						Ufact.ridx (), 
 						reinterpret_cast<double *> (Ufact.data()), 
-						NULL, 1, control);
+						0, 1, control);
 		      UMFPACK_ZNAME (report_perm) (nr, p, control);
 		      UMFPACK_ZNAME (report_perm) (nc, q, control);
 		    }
