@@ -2,7 +2,7 @@
 /*
 
 Copyright (C) 1996, 1997, 1998, 2000, 2001, 2002, 2003, 2004, 2005,
-              2006, 2007 John W. Eaton
+              2006, 2007, 2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -33,6 +33,7 @@ along with Octave; see the file COPYING.  If not, see
 #include "str-vec.h"
 #include "mx-base.h"
 #include "mx-inlines.cc"
+#include "mx-op-defs.h"
 
 // boolMatrix class.
 
@@ -77,51 +78,10 @@ boolMatrix::operator ! (void) const
 
 // other operations
 
-boolMatrix
-boolMatrix::diag (void) const
-{
-  return diag (0);
-}
-
 boolMatrix
 boolMatrix::diag (octave_idx_type k) const
 {
-  octave_idx_type nnr = rows ();
-  octave_idx_type nnc = cols ();
-  if (k > 0)
-    nnc -= k;
-  else if (k < 0)
-    nnr += k;
-
-  boolMatrix d;
-
-  if (nnr > 0 && nnc > 0)
-    {
-      octave_idx_type ndiag = (nnr < nnc) ? nnr : nnc;
-
-      d.resize (ndiag, 1);
-
-      if (k > 0)
-	{
-	  for (octave_idx_type i = 0; i < ndiag; i++)
-	    d.xelem (i) = elem (i, i+k);
-	}
-      else if (k < 0)
-	{
-	  for (octave_idx_type i = 0; i < ndiag; i++)
-	    d.xelem (i) = elem (i-k, i);
-	}
-      else
-	{
-	  for (octave_idx_type i = 0; i < ndiag; i++)
-	    d.xelem (i) = elem (i, i);
-	}
-    }
-  else
-    (*current_liboctave_error_handler)
-      ("diag: requested diagonal out of range");
-
-  return d;
+  return Array2<bool>::diag (k);
 }
 
 // FIXME Do these really belong here?  Maybe they should be
@@ -130,13 +90,13 @@ boolMatrix::diag (octave_idx_type k) const
 boolMatrix
 boolMatrix::all (int dim) const
 {
-  MX_ALL_OP (dim);
+  return do_mx_red_op<boolMatrix, bool> (*this, dim, mx_inline_all);
 }
 
 boolMatrix
 boolMatrix::any (int dim) const
 {
-  MX_ANY_OP (dim);
+  return do_mx_red_op<boolMatrix, bool> (*this, dim, mx_inline_any);
 }
 
 MM_CMP_OPS (boolMatrix, , boolMatrix, )
