@@ -1,7 +1,8 @@
 // N-D Array  manipulations.
 /*
 
-Copyright (C) 1996, 1997, 2003, 2004, 2005, 2006, 2007 John W. Eaton
+Copyright (C) 1996, 1997, 2003, 2004, 2005, 2006, 2007, 2008,
+              2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -29,6 +30,7 @@ along with Octave; see the file COPYING.  If not, see
 #include "CNDArray.h"
 #include "mx-base.h"
 #include "lo-ieee.h"
+#include "mx-op-defs.h"
 
 // unary operations
 
@@ -48,13 +50,27 @@ boolNDArray::operator ! (void) const
 boolNDArray
 boolNDArray::all (int dim) const
 {
-  MX_ND_ANY_ALL_REDUCTION (MX_ND_ALL_EVAL (MX_ND_ALL_EXPR), true);
+  return do_mx_red_op<boolNDArray, bool> (*this, dim, mx_inline_all);
 }
 
 boolNDArray
 boolNDArray::any (int dim) const
 {
-  MX_ND_ANY_ALL_REDUCTION (MX_ND_ANY_EVAL (MX_ND_ANY_EXPR), false);
+  return do_mx_red_op<boolNDArray, bool> (*this, dim, mx_inline_any);
+}
+
+NDArray 
+boolNDArray::sum (int dim) const
+{
+  // NOTE: going via octave_idx_type is faster even though it requires a conversion.
+  return do_mx_red_op<Array<octave_idx_type> , bool> (*this, dim, mx_inline_count);
+}
+
+NDArray 
+boolNDArray::cumsum (int dim) const
+{
+  // NOTE: going via octave_idx_type is faster even though it requires a conversion.
+  return do_mx_cum_op<Array<octave_idx_type> , bool> (*this, dim, mx_inline_cumcount);
 }
 
 boolNDArray
@@ -123,6 +139,12 @@ boolNDArray::compute_index (Array<octave_idx_type>& ra_idx,
   return ::compute_index (ra_idx, dimensions);
 }
 
+boolNDArray
+boolNDArray::diag (octave_idx_type k) const
+{
+  return ArrayN<bool>::diag (k);
+}
+
 NDND_BOOL_OPS (boolNDArray, boolNDArray, false)
 NDND_CMP_OPS (boolNDArray, , boolNDArray, )
 
