@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1996, 1997, 2003, 2004, 2005, 2006, 2007, 2008,
+              2009 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -29,7 +29,7 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "intNDArray.h"
 
 #include "mx-defs.h"
-#include "mx-op-defs.h"
+#include "mx-op-decl.h"
 
 class
 OCTAVE_API
@@ -48,11 +48,20 @@ public:
 
   NDArray (const Matrix& a) : MArrayN<double> (a) { }
 
-  NDArray (const MArrayN<double>& a) : MArrayN<double> (a) { }
+  NDArray (const Array<octave_idx_type>& a, bool zero_based = false, 
+	   bool negative_to_nan = false);
+
+  template <class U>
+  NDArray (const MArrayN<U>& a) : MArrayN<double> (a) { }
+
+  template <class U>
+  NDArray (const ArrayN<U>& a) : MArrayN<double> (a) { }
 
   template <class U>
   explicit NDArray (const intNDArray<U>& a) : MArrayN<double> (a) { }
 
+  NDArray (const charNDArray&); 
+
   NDArray& operator = (const NDArray& a)
     {
       MArrayN<double>::operator = (a);
@@ -64,8 +73,10 @@ public:
   boolNDArray operator ! (void) const;
 
   bool any_element_is_negative (bool = false) const;
+  bool any_element_is_nan (void) const;
   bool any_element_is_inf_or_nan (void) const;
   bool any_element_not_one_or_zero (void) const;
+  bool all_elements_are_zero (void) const;
   bool all_elements_are_int_or_inf_or_nan (void) const;
   bool all_integers (double& max_val, double& min_val) const;
   bool too_large_for_float (void) const;
@@ -89,10 +100,18 @@ public:
   NDArray min (int dim = 0) const;
   NDArray min (ArrayN<octave_idx_type>& index, int dim = 0) const;
   
+  NDArray cummax (int dim = 0) const;
+  NDArray cummax (ArrayN<octave_idx_type>& index, int dim = 0) const;
+  NDArray cummin (int dim = 0) const;
+  NDArray cummin (ArrayN<octave_idx_type>& index, int dim = 0) const;
+
   NDArray& insert (const NDArray& a, octave_idx_type r, octave_idx_type c);
   NDArray& insert (const NDArray& a, const Array<octave_idx_type>& ra_idx);
 
   NDArray abs (void) const;
+  boolNDArray isnan (void) const;
+  boolNDArray isinf (void) const;
+  boolNDArray isfinite (void) const;
 
   ComplexNDArray fourier (int dim = 1) const;
   ComplexNDArray ifourier (int dim = 1) const;
@@ -106,6 +125,8 @@ public:
   friend OCTAVE_API NDArray real (const ComplexNDArray& a);
   friend OCTAVE_API NDArray imag (const ComplexNDArray& a);
 
+  friend class ComplexNDArray;
+
   Matrix matrix_value (void) const;
 
   NDArray squeeze (void) const { return MArrayN<double>::squeeze (); }
@@ -124,6 +145,16 @@ public:
 
   static double resize_fill_value (void) { return 0; }
 
+  NDArray diag (octave_idx_type k = 0) const;
+
+  typedef double (*dmapper) (double);
+  typedef Complex (*cmapper) (const Complex&);
+  typedef bool (*bmapper) (double);
+
+  NDArray map (dmapper fcn) const;
+  ComplexNDArray map (cmapper fcn) const;
+  boolNDArray map (bmapper fcn) const;
+
 private:
 
   NDArray (double *d, const dim_vector& dv) : MArrayN<double> (d, dv) { }
@@ -142,14 +173,14 @@ extern OCTAVE_API NDArray max (double d, const NDArray& m);
 extern OCTAVE_API NDArray max (const NDArray& m, double d);
 extern OCTAVE_API NDArray max (const NDArray& a, const NDArray& b);
 
-NDS_CMP_OP_DECLS (NDArray, double)
-NDS_BOOL_OP_DECLS (NDArray, double)
+NDS_CMP_OP_DECLS (NDArray, double, OCTAVE_API)
+NDS_BOOL_OP_DECLS (NDArray, double, OCTAVE_API)
 
-SND_CMP_OP_DECLS (double, NDArray)
-SND_BOOL_OP_DECLS (double, NDArray)
+SND_CMP_OP_DECLS (double, NDArray, OCTAVE_API)
+SND_BOOL_OP_DECLS (double, NDArray, OCTAVE_API)
 
-NDND_CMP_OP_DECLS (NDArray, NDArray)
-NDND_BOOL_OP_DECLS (NDArray, NDArray)
+NDND_CMP_OP_DECLS (NDArray, NDArray, OCTAVE_API)
+NDND_BOOL_OP_DECLS (NDArray, NDArray, OCTAVE_API)
 
 MARRAY_FORWARD_DEFS (MArrayN, NDArray, double)
 
