@@ -1,4 +1,4 @@
-## Copyright (C) 1995, 1996, 1997, 2005, 2007 Kurt Hornik
+## Copyright (C) 1995, 1996, 1997, 2005, 2007, 2008, 2009 Kurt Hornik
 ##
 ## This file is part of Octave.
 ##
@@ -24,15 +24,24 @@
 ## or of size @var{r} by @var{c}.
 ##
 ## If @var{r} and @var{c} are given create a matrix with @var{r} rows and
-## @var{c} columns. Or if @var{sz} is a vector, create a matrix of size
+## @var{c} columns.  Or if @var{sz} is a vector, create a matrix of size
 ## @var{sz}.
 ## @end deftypefn
 
 ## Author: KH <Kurt.Hornik@wu-wien.ac.at>
 ## Description: Random deviates from the geometric distribution
 
+## Deprecated in version 3.0
+
 function rnd = geometric_rnd (varargin)
 
+  persistent warned = false;
+  if (! warned)
+    warned = true;
+    warning ("Octave:deprecated-function",
+             "geometric_rnd is obsolete and will be removed from a future version of Octave; please use geornd instead");
+  endif
+
  rnd =  geornd (varargin{:});
 
 endfunction
