@@ -1,4 +1,4 @@
-## Copyright (C) 1995, 1996, 1997, 2005, 2007 Kurt Hornik
+## Copyright (C) 1995, 1996, 1997, 2005, 2007, 2008 Kurt Hornik
 ##
 ## This file is part of Octave.
 ##
@@ -30,8 +30,17 @@
 ## Author: KH <Kurt.Hornik@wu-wien.ac.at>
 ## Description: Quantile function of the t distribution
 
+## Deprecated in version 3.0
+
 function inv = t_inv (varargin)
 
+  persistent warned = false;
+  if (! warned)
+    warned = true;
+    warning ("Octave:deprecated-function",
+             "t_inv is obsolete and will be removed from a future version of Octave; please use tinv instead");
+  endif
+
  inv =  tinv (varargin{:});
 
 endfunction
