@@ -1,4 +1,4 @@
-## Copyright (C) 2000, 2006, 2007 Paul Kienzle
+## Copyright (C) 2000, 2006, 2007, 2008, 2009 Paul Kienzle
 ##
 ## This file is part of Octave.
 ##
@@ -22,22 +22,22 @@
 ## @deftypefnx {Function File} {@var{yi} =} interp1 (@dots{}, @var{extrap})
 ## @deftypefnx {Function File} {@var{pp} =} interp1 (@dots{}, 'pp')
 ##
-## One-dimensional interpolation. Interpolate @var{y}, defined at the
-## points @var{x}, at the points @var{xi}. The sample points @var{x} 
-## must be strictly monotonic. If @var{y} is an array, treat the columns
+## One-dimensional interpolation.  Interpolate @var{y}, defined at the
+## points @var{x}, at the points @var{xi}.  The sample points @var{x} 
+## must be strictly monotonic.  If @var{y} is an array, treat the columns
 ## of @var{y} separately.
 ##
 ## Method is one of:
 ##
 ## @table @asis
 ## @item 'nearest'
-## Return the nearest neighbour.
+## Return the nearest neighbor.
 ## @item 'linear'
-## Linear interpolation from nearest neighbours
+## Linear interpolation from nearest neighbors
 ## @item 'pchip'
 ## Piece-wise cubic hermite interpolating polynomial
 ## @item 'cubic'
-## Cubic interpolation from four nearest neighbours
+## Cubic interpolation from four nearest neighbors
 ## @item 'spline'
 ## Cubic spline interpolation--smooth first and second derivatives
 ## throughout the curve
@@ -45,8 +45,8 @@
 ##
 ## Appending '*' to the start of the above method forces @code{interp1}
 ## to assume that @var{x} is uniformly spaced, and only @code{@var{x}
-## (1)} and @code{@var{x} (2)} are referenced. This is usually faster,
-## and is never slower. The default method is 'linear'.
+## (1)} and @code{@var{x} (2)} are referenced.  This is usually faster,
+## and is never slower.  The default method is 'linear'.
 ##
 ## If @var{extrap} is the string 'extrap', then extrapolate values beyond
 ## the endpoints.  If @var{extrap} is a number, replace values beyond the
@@ -63,15 +63,17 @@
 ##
 ## @example
 ## @group
-##    xf=[0:0.05:10]; yf = sin(2*pi*xf/5);
-##    xp=[0:10];      yp = sin(2*pi*xp/5);
-##    lin=interp1(xp,yp,xf);
-##    spl=interp1(xp,yp,xf,'spline');
-##    cub=interp1(xp,yp,xf,'cubic');
-##    near=interp1(xp,yp,xf,'nearest');
-##    plot(xf,yf,"r",xf,lin,"g",xf,spl,"b", ...
-##         xf,cub,"c",xf,near,"m",xp,yp,"r*");
-##    legend ("original","linear","spline","cubic","nearest")
+## xf = [0:0.05:10];
+## yf = sin (2*pi*xf/5);
+## xp = [0:10];
+## yp = sin (2*pi*xp/5);
+## lin = interp1 (xp, yp, xf);
+## spl = interp1 (xp, yp, xf, "spline");
+## cub = interp1 (xp, yp, xf, "cubic");
+## near = interp1 (xp, yp, xf, "nearest");
+## plot (xf, yf, "r", xf, lin, "g", xf, spl, "b",
+##       xf, cub, "c", xf, near, "m", xp, yp, "r*");
+## legend ("original", "linear", "spline", "cubic", "nearest")
 ## @end group
 ## @end example
 ##
@@ -142,7 +144,7 @@ function yi = interp1 (x, y, varargin)
   endif
 
   ## determine which values are out of range and set them to extrap,
-  ## unless extrap=="extrap" in which case, extrapolate them like we
+  ## unless extrap == "extrap" in which case, extrapolate them like we
   ## should be doing in the first place.
   minx = x(1);
   maxx = x(nx);
@@ -200,9 +202,7 @@ function yi = interp1 (x, y, varargin)
       yi = mkpp (x, [dy./dx, y(1:end-1)], szy(2:end));
     else
       ## find the interval containing the test point
-      idx = lookup (x(2:nx-1), xi)+1;
-				# 2:n-1 so that anything beyond the ends
-				# gets dumped into an interval
+      idx = lookup (x, xi, "lr");
       ## use the endpoints of the interval to define a line
       s = (xi - x(idx))./dx(idx);
       yi(range,:) = s(:,ones(1,nc)).*dy(idx,:) + y(idx,:);
@@ -243,7 +243,7 @@ function yi = interp1 (x, y, varargin)
     if (nx < 4 || ny < 4)
       error ("interp1: table too short");
     endif
-    idx = lookup (x(3:nx-2), xi) + 1;
+    idx = lookup (x(2:nx-1), xi, "lr");
 
     ## Construct cubic equations for each interval using divided
     ## differences (computation of c and d don't use divided differences
@@ -570,3 +570,5 @@ endfunction
 %!assert (interp1(1:2:8,1:2:8,1.4,"*cubic"),1.4);
 %!error interp1(1:2,1:2,1, "*spline");
 %!assert (interp1(1:2:6,1:2:6,1.4,"*spline"),1.4);
+
+%!assert (interp1([3,2,1],[3,2,2],2.5),2.5)
