@@ -1,4 +1,4 @@
-## Copyright (C) 2006, 2007 Paul Kienzle
+## Copyright (C) 2006, 2007, 2009 Paul Kienzle
 ##
 ## This file is part of Octave.
 ##
@@ -90,13 +90,47 @@ function [t, p] = orderfields (s1, s2)
   if (numel (s1) == 0)
     args = cell (1, 2 * numel (names));
     args(1:2:end) = names;
-      args{2:2:end} = {};
+    args(2:2:end) = {[]};
     t = struct (args{:});
   else
+    n = numel (s1);
     for i = 1:numel (names)
       el = names(i);
-      t(:).(el) = s1(:).(el);
+      [t(1:n).(el)] = s1(:).(el);
     endfor
+    ## inherit dimensions
+    t = reshape (t, size (s1));
   endif
 
 endfunction
+
+%!shared a, b, c
+%! a = struct ("foo", {1, 2}, "bar", {3, 4});
+%! b = struct ("bar", 6, "foo", 5);
+%! c = struct ("bar", {7, 8}, "foo", 9);
+%!test
+%! a(2) = orderfields (b, a);
+%! assert (a(2).foo, 5)
+%! assert (a(2).bar, 6)
+%!test
+%! a(2) = orderfields (b, [2 1]);
+%! assert (a(2).foo, 5)
+%! assert (a(2).bar, 6)
+%!test
+%! a(2) = orderfields (b, fieldnames (a));
+%! assert (a(2).foo, 5)
+%! assert (a(2).bar, 6)
+%!test
+%! a(1:2) = orderfields (c, fieldnames (a));
+%! assert (a(2).foo, 9)
+%! assert (a(2).bar, 8)
+
+%!test
+%! aa.x = {1, 2};
+%! aa.y = 3;
+%! aa(2).x = {4, 5};
+%! bb.y = {6, 7};
+%! bb.x = 8;
+%! aa(2) = orderfields (bb, aa);
+%! assert (aa(2).x, 8);
+%! assert (aa(2).y{1}, 6);
