@@ -1,5 +1,5 @@
 ## Copyright (C) 1994, 1995, 1996, 1997, 1999, 2000, 2002, 2005, 2006,
-##               2007 John W. Eaton
+##               2007, 2009 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -17,7 +17,10 @@
 ## along with Octave; see the file COPYING.  If not, see
 ## <http://www.gnu.org/licenses/>.
 
+## -*- texinfo -*-
+## @deftypefn {Function File} {} __plr2__ (@var{h}, @var{theta}, @var{rho}, @var{fmt})
 ## Undocumented internal function.
+## @end deftypefn
 
 ## Author: jwe
 
@@ -79,7 +82,7 @@ function retval = __plr2__ (h, theta, rho, fmt)
       y = diag (sin (theta)) * rho;
       retval = __plt__ ("polar", h, x, y, fmt);
     else
-      error ("__plr2__: invalid data for plotting")
+      error ("__plr2__: invalid data for plotting");
     endif
   elseif (ismatrix (theta))
     if (isvector (rho))
@@ -112,10 +115,10 @@ function retval = __plr2__ (h, theta, rho, fmt)
       y = rho .* sin (theta);
       retval = __plt__ ("polar", h, x, y, fmt);
     else
-      error ("__plr2__: invalid data for plotting")
+      error ("__plr2__: invalid data for plotting");
     endif
   else
-    error ("__plr2__: invalid data for plotting")
+    error ("__plr2__: invalid data for plotting");
   endif
 
 endfunction
