@@ -1,4 +1,4 @@
-## Copyright (C) 2004, 2005, 2007 David Bateman & Andy Adler
+## Copyright (C) 2004, 2005, 2007, 2008, 2009 David Bateman and Andy Adler
 ##
 ## This file is part of Octave.
 ##
@@ -18,7 +18,7 @@
 
 ## -*- texinfo -*-
 ## @deftypefn {Function File} {@var{y} =} spones (@var{x})
-## Replace the non-zero entries of @var{x} with ones. This creates a
+## Replace the non-zero entries of @var{x} with ones.  This creates a
 ## sparse matrix with the same structure as @var{x}.
 ## @end deftypefn
 
@@ -28,12 +28,8 @@ function s = spones (s)
     print_usage ();
   endif
 
-  if (issparse (s))
-    [i, j, v, m, n] = spfind (s);
-  else
-    [i, j, v] = find (s);
-    [m, n] = size (s);
-  endif
+  [i, j, v] = find (s);
+  [m, n] = size (s);
 
   s = sparse (i, j, 1, m, n);
 
