@@ -1,4 +1,4 @@
-## Copyright (C) 2005, 2006, 2007 Paul Kienzle
+## Copyright (C) 2005, 2006, 2007, 2008, 2009 Paul Kienzle
 ##
 ## This file is part of Octave.
 ##
@@ -26,7 +26,7 @@
 ## @deftypefnx {Function File} {[@var{code}, @var{idx}] =} test ('@var{name}','grabdemo')
 ##
 ## Perform tests from the first file in the loadpath matching @var{name}.
-## @code{test} can be called as a command or as a function. Called with 
+## @code{test} can be called as a command or as a function.  Called with 
 ## a single argument @var{name}, the tests are run interactively and stop
 ## after the first error is encountered.
 ##
@@ -45,21 +45,21 @@
 ## Do tests which require user interaction.
 ## @end table
 ##
-## The argument @var{fid} can be used to allow batch processing. Errors
+## The argument @var{fid} can be used to allow batch processing.  Errors
 ## can be written to the already open file defined by @var{fid}, and 
-## hopefully when octave crashes this file will tell you what was happening
-## when it did. You can use @code{stdout} if you want to see the results as
+## hopefully when Octave crashes this file will tell you what was happening
+## when it did.  You can use @code{stdout} if you want to see the results as
 ## they happen.  You can also give a file name rather than an @var{fid}, in
 ## which case the contents of the file will be replaced with the log from 
 ## the current test.
 ##
 ## Called with a single output argument @var{success}, @code{test} returns
-## true if all of the tests were successful. Called with two output arguments
+## true if all of the tests were successful.  Called with two output arguments
 ## @var{n} and @var{max}, the number of successful tests and the total number
 ## of tests in the file @var{name} are returned.
 ##
 ## If the second argument is the string 'grabdemo', the contents of the demo
-## blocks are extracted but not executed. Code for all code blocks is
+## blocks are extracted but not executed.  Code for all code blocks is
 ## concatenated and returned as @var{code} with @var{idx} being a vector of
 ## positions of the ends of the demo blocks.
 ##
@@ -68,21 +68,21 @@
 ## @seealso{error, assert, fail, demo, example}
 ## @end deftypefn
 
-## TODO: * Consider using keyword fail rather then error?  This allows us
-## TODO: to make a functional form of error blocks, which means we
-## TODO: can include them in test sections which means that we can use
-## TODO: octave flow control for both kinds of tests.
+## FIXME: * Consider using keyword fail rather then error?  This allows us
+## to make a functional form of error blocks, which means we
+## can include them in test sections which means that we can use
+## octave flow control for both kinds of tests.
 
-## PKG_ADD: mark_as_command test
-
-function [__ret1, __ret2, __ret3] = test (__name, __flag, __fid)
-  ## information from test will be introduced by "key" 
+function [__ret1, __ret2, __ret3, __ret4] = test (__name, __flag, __fid)
+  ## Information from test will be introduced by "key".
   persistent __signal_fail =  "!!!!! ";
   persistent __signal_empty = "????? ";
   persistent __signal_block = "  ***** ";
   persistent __signal_file =  ">>>>> ";
+  persistent __signal_skip = "----- ";
 
   __xfail = 0;
+  __xskip = 0;
 
   if (nargin < 2 || isempty (__flag))
     __flag = "quiet";
@@ -99,7 +99,7 @@ function [__ret1, __ret2, __ret3] = test (__name, __flag, __fid)
   endif
   __batch = (! isempty (__fid));
 
-  ## decide if error messages should be collected
+  ## Decide if error messages should be collected.
   __close_fid = 0;
   if (__batch)
     if (ischar (__fid))
@@ -152,13 +152,13 @@ function [__ret1, __ret2, __ret3] = test (__name, __flag, __fid)
     error ("test unknown flag '%s'", __flag);
   endif
 
-  ## locate the file to test
+  ## Locate the file to test.
   __file = file_in_loadpath (__name, "all");
   if (isempty (__file))
-    __file = file_in_loadpath (strcat (__name, ".m"), "all");
+    __file = file_in_loadpath (cstrcat (__name, ".m"), "all");
   endif
   if (isempty (__file))
-    __file = file_in_loadpath (strcat (__name, ".cc"), "all");
+    __file = file_in_loadpath (cstrcat (__name, ".cc"), "all");
   endif
   if (iscell (__file))
       ## If repeats, return first in path.
@@ -185,7 +185,7 @@ function [__ret1, __ret2, __ret3] = test (__name, __flag, __fid)
     return;
   endif
 
-  ## grab the test code from the file
+  ## Grab the test code from the file.
   __body = __extract_test_code (__file);
 
   if (isempty (__body))
@@ -204,7 +204,7 @@ function [__ret1, __ret2, __ret3] = test (__name, __flag, __fid)
     endif
     return;
   else
-    ## add a dummy comment block to the end for ease of indexing
+    ## Add a dummy comment block to the end for ease of indexing.
     if (__body (length(__body)) == "\n")
       __body = sprintf ("\n%s#", __body); 
     else
@@ -212,35 +212,35 @@ function [__ret1, __ret2, __ret3] = test (__name, __flag, __fid)
     endif
   endif
 
-  ## chop it up into blocks for evaluation
+  ## Chop it up into blocks for evaluation.
   __lineidx = find (__body == "\n");
   __blockidx = __lineidx(find (! isspace (__body(__lineidx+1))))+1;
 
-  ## ready to start tests ... if in batch mode, tell us what is happening
+  ## Ready to start tests ... if in batch mode, tell us what is happening.
   if (__verbose)
-    disp (strcat ( __signal_file, __file));
+    disp (cstrcat (__signal_file, __file));
   endif
 
-  ## assume all tests will pass
+  ## Assume all tests will pass.
   __all_success = 1;
 
-  ## process each block separately, initially with no shared variables
+  ## Process each block separately, initially with no shared variables.
   __tests = __successes = 0;
   __shared = " ";
   __shared_r = " ";
   __clear = "";
   for __i = 1:length(__blockidx)-1
 
-    ## extract the block
+    ## Extract the block.
     __block = __body(__blockidx(__i):__blockidx(__i+1)-2);
 
-    ## let the user/logfile know what is happening
+    ## Let the user/logfile know what is happening.
     if (__verbose)
       fprintf (__fid, "%s%s\n", __signal_block, __block);
       fflush (__fid);
     endif
 
-    ## split __block into __type and __code
+    ## Split __block into __type and __code.
     __idx = find (! isletter (__block));
     if (isempty (__idx))
       __type = __block;
@@ -250,15 +250,17 @@ function [__ret1, __ret2, __ret3] = test (__name, __flag, __fid)
       __code = __block(__idx(1):length(__block));
     endif
 
-    ## assume the block will succeed;
+    ## Assume the block will succeed.
     __success = 1;
     __msg = [];
 
-    ## DEMO
+### DEMO
+
     ## If in __grabdemo mode, then don't process any other block type.
     ## So that the other block types don't have to worry about
     ## this __grabdemo mode, the demo block processor grabs all block
     ## types and skips those which aren't demo blocks.
+
     __isdemo = strcmp (__type, "demo");
     if (__grabdemo || __isdemo)
       __istest = 0;
@@ -268,7 +270,7 @@ function [__ret1, __ret2, __ret3] = test (__name, __flag, __fid)
 	  __demo_code = __code;
 	  __demo_idx = [1, length(__demo_code)+1];
 	else
-	  __demo_code = strcat(__demo_code, __code);
+	  __demo_code = cstrcat(__demo_code, __code);
 	  __demo_idx = [__demo_idx, length(__demo_code)+1];
 	endif
 
@@ -285,13 +287,15 @@ function [__ret1, __ret2, __ret3] = test (__name, __flag, __fid)
       	clear __test__;
 
       endif
-      __code = ""; # code already processed
+      ## Code already processed.
+      __code = "";
       
-    ## SHARED
-    elseif strcmp (__type, "shared")
+### SHARED
+
+    elseif (strcmp (__type, "shared"))
       __istest = 0;
 
-      ## separate initialization code from variables
+      ## Separate initialization code from variables.
       __idx = find (__code == "\n");
       if (isempty (__idx))
 	__vars = __code;
@@ -301,37 +305,39 @@ function [__ret1, __ret2, __ret3] = test (__name, __flag, __fid)
       	__code = __code (__idx(1):length(__code));
       endif
       
-      ## strip comments off the variables
+      ## Strip comments off the variables.
       __idx = find (__vars == "%" | __vars == "#");
       if (! isempty (__idx))
 	__vars = __vars(1:__idx(1)-1);
       endif
       
-      ## assign default values to variables
+      ## Assign default values to variables.
       try
 	__vars = deblank (__vars);
 	if (! isempty (__vars))
-	  eval (strcat (strrep (__vars, ",", "=[];"), "=[];"));
+	  eval (cstrcat (strrep (__vars, ",", "=[];"), "=[];"));
 	  __shared = __vars;
-	  __shared_r = strcat ("[ ", __vars, "] = ");
+	  __shared_r = cstrcat ("[ ", __vars, "] = ");
       	else
 	  __shared = " ";
 	  __shared_r = " ";
       	endif
       catch
-	__code = "";  # couldn't declare, so don't initialize
+	## Couldn't declare, so don't initialize.
+	__code = "";
 	__success = 0;
 	__msg = sprintf ("%sshared variable initialization failed\n",
 		         __signal_fail);
       end_try_catch
 
-      ## clear shared function definitions
+      ## Clear shared function definitions.
       eval (__clear, "");
       __clear = "";
       
-      ## initialization code will be evaluated below
+      ## Initialization code will be evaluated below.
     
-    ## FUNCTION
+### FUNCTION
+
     elseif (strcmp (__type, "function"))
       __istest = 0;
       persistent __fn = 0;
@@ -354,14 +360,16 @@ function [__ret1, __ret2, __ret3] = test (__name, __flag, __fid)
       endif
       __code = "";
       
+### ASSERT/FAIL
 
-    ## ASSERT/FAIL
     elseif (strcmp (__type, "assert") || strcmp (__type, "fail"))
       __istest = 1;
-      __code = __block; # put the keyword back on the code
-      ## the code will be evaluated below as a test block
+      ## Put the keyword back on the code.
+      __code = __block;
+      ## The code will be evaluated below as a test block.
       
-    ## ERROR/WARNING
+### ERROR/WARNING
+
     elseif (strcmp (__type, "error") || strcmp(__type, "warning"))
       __istest = 1;
       __warning = strcmp (__type, "warning");
@@ -413,20 +421,40 @@ function [__ret1, __ret2, __ret3] = test (__name, __flag, __fid)
       	end_try_catch
       	clear __test__;
       endif
-      __code = ""; # code already processed
+      ## Code already processed.
+      __code = "";
       
-    ## TEST
+### TESTIF
+
+    elseif (strcmp (__type, "testif"))
+      [__e, __feat] = regexp (__code, '^\s*([^\s]+)', 'end', 'tokens');
+      if (isempty (findstr (octave_config_info ("DEFS"), __feat{1}{1})))
+        __xskip++;
+	__success = 0;
+	__istest = 0;
+	## Skip the code.
+	__code = "";
+	__msg = sprintf ("%sskipped test\n", __signal_skip);
+      else
+        __istest = 1;
+	__code = __code(__e + 1 : end);
+      endif
+
+### TEST
+
     elseif (strcmp (__type, "test") || strcmp (__type, "xtest"))
       __istest = 1;
-      ## code will be evaluated below
-      
-    ## comment block
+      ## Code will be evaluated below.
+
+### Comment block.
+
     elseif (strcmp (__block(1:1), "#"))
       __istest = 0;
       __code = ""; # skip the code
 
+### Unknown block.
+
     else
-    ## unknown block
       __istest = 1;
       __success = 0;
       __msg = sprintf ("%sunknown test type!\n", __signal_fail);
@@ -454,17 +482,18 @@ function [__ret1, __ret2, __ret3] = test (__name, __flag, __fid)
       clear __test__;
     endif
     
-    ## All done.  Remember if we were successful and print any messages
+    ## All done.  Remember if we were successful and print any messages.
     if (! isempty (__msg))
-      ## make sure the user knows what caused the error
+      ## Make sure the user knows what caused the error.
       if (! __verbose)
       	fprintf (__fid, "%s%s\n", __signal_block, __block);
 	fflush (__fid);
       endif
       fputs (__fid, __msg);
       fflush (__fid);
-      ## show the variable context
-      if (! strcmp (__type, "error") && ! all (__shared == " "))
+      ## Show the variable context.
+      if (! strcmp (__type, "error") && ! strcmp (__type, "testif")
+	  && ! all (__shared == " "))
 	fputs (__fid, "shared variables ");
 	eval (sprintf ("fdisp(__fid,bundle(%s));", __shared)); 
 	fflush (__fid);
@@ -472,7 +501,7 @@ function [__ret1, __ret2, __ret3] = test (__name, __flag, __fid)
     endif
     if (__success == 0)
       __all_success = 0;
-      	## stop after one error if not in batch mode
+      ## Stop after one error if not in batch mode.
       if (! __batch)
     	if (nargout > 0)
 	  __ret1 = __ret2 = 0;
@@ -489,11 +518,18 @@ function [__ret1, __ret2, __ret3] = test (__name, __flag, __fid)
   eval (__clear, "");
 
   if (nargout == 0)
-    if (__xfail)
-      printf ("PASSES %d out of %d tests (%d expected failures)\n",
-	      __successes, __tests, __xfail);
+    if (__tests || __xfail || __xskip)
+      if (__xfail)
+	printf ("PASSES %d out of %d tests (%d expected failures)\n",
+		__successes, __tests, __xfail);
+      else
+	printf ("PASSES %d out of %d tests\n", __successes, __tests);
+      endif
+      if (__xskip)
+	printf ("Skipped %d tests due to missing features\n", __xskip);
+      endif
     else
-      printf ("PASSES %d out of %d tests\n", __successes, __tests);
+      printf ("%s%s has no tests available\n", __signal_empty, __file);
     endif
   elseif (__grabdemo)
     __ret1 = __demo_code;
@@ -504,28 +540,29 @@ function [__ret1, __ret2, __ret3] = test (__name, __flag, __fid)
     __ret1 = __successes;
     __ret2 = __tests;
     __ret3 = __xfail;
+    __ret4 = __xskip;
   endif
 endfunction
 
-## create structure with fieldnames the name of the input variables
+## Create structure with fieldnames the name of the input variables.
 function s = varstruct (varargin)
   for i = 1:nargin
     s.(deblank (argn(i,:))) = varargin{i};
   endfor
 endfunction
 
-## find [start,end] of fn in 'function [a,b] = fn'
+## Find [start,end] of fn in 'function [a,b] = fn'.
 function pos = function_name (def)
   pos = [];
 
-  ## Find the end of the name
+  ## Find the end of the name.
   right = find (def == "(", 1);
   if (isempty (right))
     return;
   endif
   right = find (def(1:right-1) != " ", 1, "last");
 
-  ## Find the beginning of the name
+  ## Find the beginning of the name.
   left = max ([find(def(1:right)==" ", 1, "last"), ...
 	       find(def(1:right)=="=", 1, "last")]);
   if (isempty (left))
@@ -533,11 +570,11 @@ function pos = function_name (def)
   endif
   left++;
 
-  ## Return the end points of the name
+  ## Return the end points of the name.
   pos = [left, right];
 endfunction
 
-## strip <pattern> from '<pattern> code'
+## Strip <pattern> from '<pattern> code'.
 function [pattern, rest] = getpattern (str)
   pattern = ".";
   rest = str; 
@@ -551,16 +588,16 @@ function [pattern, rest] = getpattern (str)
   endif
 endfunction
 
-## strip '.*prefix:' from '.*prefix: msg\n' and strip trailing blanks
+## Strip '.*prefix:' from '.*prefix: msg\n' and strip trailing blanks.
 function msg = trimerr (msg, prefix)
-  idx = index (msg, strcat (prefix, ":"));
+  idx = index (msg, cstrcat (prefix, ":"));
   if (idx > 0)
     msg(1:idx+length(prefix)) = [];
   endif
   msg = trimleft (deblank (msg));
 endfunction
 
-## strip leading blanks from string
+## Strip leading blanks from string.
 function str = trimleft (str)
   idx = find (isspace (str));
   leading = find (idx == 1:length(idx));
@@ -569,8 +606,8 @@ function str = trimleft (str)
   endif
 endfunction
 
-## make a structure out of the named variables
-## (based on Etienne Grossmann's tar function)
+## Make a structure out of the named variables
+## (based on Etienne Grossmann's tar function).
 function s = bundle (varargin)
   for i = 1:nargin
     s.(deblank (argn(i,:))) = varargin{i};
@@ -586,7 +623,7 @@ function body = __extract_test_code (nm)
       if (length (ln) >= 2 && strcmp (ln(1:2), "%!"))
         body = [body, "\n"];
         if (length(ln) > 2)
-          body = strcat (body, ln(3:end));
+          body = cstrcat (body, ln(3:end));
         endif
       endif
     endwhile
@@ -594,6 +631,16 @@ function body = __extract_test_code (nm)
   endif
 endfunction
 
+### Test for test for missing features
+%!testif OCTAVE_SOURCE
+%! ## This test should be run
+%! assert (true);
+
+### Disable this test to avoid spurious skipped test for "make check"
+% !testif HAVE_FOOBAR
+% ! ## missing feature. Fail if this test is run
+% ! error("Failed missing feature test");
+
 ### Test for a known failure
 %!xtest error("This test is known to fail")
 
@@ -658,8 +705,8 @@ endfunction
 %!fail ('test("test", "bogus")','unknown flag')      # incorrect args
 %!fail ('garbage','garbage.*undefined')  # usage on nonexistent function should be
 
-%!error <usage.*test> test                     # no args, generates usage()
-%!error <usage.*test> test(1,2,3,4)            # too many args, generates usage()
+%!error test                     # no args, generates usage()
+%!error test(1,2,3,4)            # too many args, generates usage()
 %!error <unknown flag> test("test", 'bogus');  # incorrect args, generates error()
 %!error <garbage' undefined> garbage           # usage on nonexistent function should be
 
