@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2005, 2006, 2007 David Bateman
+Copyright (C) 2005, 2006, 2007, 2008, 2009 David Bateman
 Copyright (C) 2002, 2003, 2004, 2005 Paul Kienzle
 
 This file is part of Octave.
@@ -39,6 +39,7 @@ along with Octave; see the file COPYING.  If not, see
 #include "str-vec.h"
 #include "quit.h"
 #include "parse.h"
+#include "oct-locbuf.h"
 
 #if defined (HAVE_PCRE)
 #include <pcre.h>
@@ -52,6 +53,10 @@ along with Octave; see the file COPYING.  If not, see
 #include <regex.h>
 #endif
 
+// Define the maximum number of retries for a pattern that 
+// possibly results in an infinite recursion.
+#define PCRE_MATCHLIMIT_MAX 10
+
 // The regexp is constructed as a linked list to avoid resizing the
 // return values in arrays at each new match.
 
@@ -80,6 +85,9 @@ public:
 
 typedef std::list<regexp_elem>::const_iterator const_iterator;
 
+#define MAXLOOKBEHIND 10
+static bool lookbehind_warned = false;
+
 static int
 octregexp_list (const octave_value_list &args, const std::string &nm, 
 		bool case_insensitive, std::list<regexp_elem> &lst, 
@@ -96,6 +104,9 @@ octregexp_list (const octave_value_list &args, const std::string &nm,
   once = false;
 
   std::string buffer = args(0).string_value ();
+  size_t max_length = (buffer.length () > MAXLOOKBEHIND ? 
+		       MAXLOOKBEHIND: buffer.length ());
+
   if (error_state)
     {
       gripe_wrong_type_arg (nm.c_str(), args(0));
@@ -190,12 +201,6 @@ octregexp_list (const octave_value_list &args, const std::string &nm,
 
       // named tokens "(?<name>...)" are only treated with PCRE not regex.
 #if HAVE_PCRE
-      // The syntax of named tokens in pcre is "(?P<name>...)" while we need
-      // a syntax "(?<name>...)", so fix that here. Also an expression like
-      // "(?<first>\w+)\s+(?<last>\w+)|(?<last>\w+),\s+(?<first>\w+)" should
-      // be perfectly legal, while pcre does not allow the same named token
-      // name on both sides of the alternative. Also fix that here by replacing
-      // name tokens by dummy names, and dealing with the dummy names later.
       
       size_t pos = 0;
       size_t new_pos;
@@ -204,44 +209,131 @@ octregexp_list (const octave_value_list &args, const std::string &nm,
       std::ostringstream buf;
       Array<int> named_idx;
 
-      while ((new_pos = pattern.find ("(?<",pos)) != NPOS)
+      while ((new_pos = pattern.find ("(?",pos)) != std::string::npos)
 	{
-	  size_t tmp_pos = pattern.find_first_of ('>',new_pos);
-
-	  if (tmp_pos == NPOS)
+	  if (pattern.at (new_pos + 2) == '<' &&  
+	      !(pattern.at (new_pos + 3) == '=' ||
+		pattern.at (new_pos + 3) == '!'))
 	    {
-	      error ("syntax error in pattern");
-	      break;
+	      // The syntax of named tokens in pcre is "(?P<name>...)" while
+	      // we need a syntax "(?<name>...)", so fix that here. Also an 
+	      // expression like 
+	      // "(?<first>\w+)\s+(?<last>\w+)|(?<last>\w+),\s+(?<first>\w+)" 
+	      // should be perfectly legal, while pcre does not allow the same
+	      // named token name on both sides of the alternative. Also fix
+	      // that here by replacing name tokens by dummy names, and dealing
+	      // with the dummy names later.
+
+	      size_t tmp_pos = pattern.find_first_of ('>',new_pos);
+
+	      if (tmp_pos == std::string::npos)
+		{
+		  error ("syntax error in pattern");
+		  break;
+		}
+
+	      std::string tmp_name = 
+		pattern.substr(new_pos+3,tmp_pos-new_pos-3);
+	      bool found = false;
+
+	      for (int i = 0; i < nnames; i++)
+		if (named(i) == tmp_name)
+		  {
+		    named_idx.resize(inames+1);
+		    named_idx(inames) = i;
+		    found = true;
+		    break;
+		  }
+	      if (! found)
+		{
+		  named_idx.resize(inames+1);
+		  named_idx(inames) = nnames;
+		  named.append(tmp_name);
+		  nnames++;
+		}
+
+	      if (new_pos - pos > 0)
+		buf << pattern.substr(pos,new_pos-pos);
+	      if (inames < 10)
+		buf << "(?P<n00" << inames++;
+	      else if (inames < 100)
+		buf << "(?P<n0" << inames++;
+	      else
+		buf << "(?P<n" << inames++;
+	      pos = tmp_pos;
 	    }
-
-	  std::string tmp_name = pattern.substr(new_pos+3,tmp_pos-new_pos-3);
-	  bool found = false;
-
-	  for (int i = 0; i < nnames; i++)
-	    if (named(i) == tmp_name)
-	      {
-		named_idx.resize(inames+1);
-		named_idx(inames) = i;
-		found = true;
-		break;
-	      }
-	  if (! found)
+	  else if (pattern.at (new_pos + 2) == '<')
 	    {
-	      named_idx.resize(inames+1);
-	      named_idx(inames) = nnames;
-	      named.append(tmp_name);
-	      nnames++;
-	    }
+	      // Find lookbehind operators of arbitrary length (ie like 
+	      // "(?<=[a-z]*)") and replace with a maximum length operator 
+	      // as PCRE can not yet handle arbitrary length lookahead 
+	      // operators. Use the string length as the maximum length to 
+	      // avoid issues.
 
-	  if (new_pos - pos > 0)
-	    buf << pattern.substr(pos,new_pos-pos);
-	  if (inames < 10)
-	    buf << "(?P<n00" << inames++;
-	  else if (inames < 100)
-	    buf << "(?P<n0" << inames++;
+	      int brackets = 1;
+	      size_t tmp_pos1 = new_pos + 2;
+	      size_t tmp_pos2 = tmp_pos1;
+	      while (tmp_pos1 <= pattern.length () && brackets > 0)
+		{
+		  char ch = pattern.at (tmp_pos1);
+		  if (ch == '(')
+		    brackets++;
+		  else if (ch == ')')
+		    {
+		      if (brackets > 1)
+			tmp_pos2 = tmp_pos1;
+
+		      brackets--;
+		    }
+		  tmp_pos1++;
+		}
+
+	      if (brackets != 0)
+		{
+		  buf << pattern.substr (pos, new_pos - pos) << "(?";
+		  pos = new_pos + 2;
+		}
+	      else
+		{
+		  size_t tmp_pos3 = pattern.find_first_of ("*+", tmp_pos2);
+		  if (tmp_pos3 != std::string::npos && tmp_pos3 < tmp_pos1)
+		    {
+		      if (!lookbehind_warned)
+			{
+			  lookbehind_warned = true;
+			  warning ("%s: arbitrary length lookbehind patterns are only support up to length %d", nm.c_str(), MAXLOOKBEHIND);
+			}
+
+		      buf << pattern.substr (pos, new_pos - pos) << "(";
+
+		      size_t i;
+		      if (pattern.at (tmp_pos3) == '*')
+			i = 0;
+		      else
+			i = 1;
+
+		      for (; i < max_length + 1; i++)
+			{
+			  buf << pattern.substr(new_pos, tmp_pos3 - new_pos)
+			      << "{" << i << "}";
+			  buf << pattern.substr(tmp_pos3 + 1, 
+						tmp_pos1 - tmp_pos3 - 1);
+			  if (i != max_length)
+			    buf << "|";
+			}
+		      buf << ")";
+		    }
+		  else
+		    buf << pattern.substr (pos, tmp_pos1 - pos);
+		  pos = tmp_pos1;
+		}
+	    }
 	  else
-	    buf << "(?P<n" << inames++;
-	  pos = tmp_pos;
+	    {
+	      buf << pattern.substr (pos, new_pos - pos) << "(?";
+	      pos = new_pos + 2;
+	    }
+
 	}
 
       buf << pattern.substr(pos);
@@ -259,9 +351,9 @@ octregexp_list (const octave_value_list &args, const std::string &nm,
 			 (dotexceptnewline ? 0 : PCRE_DOTALL) |
 			 (lineanchors ? PCRE_MULTILINE : 0) |
 			 (freespacing ? PCRE_EXTENDED : 0),
-			 &err, &erroffset, NULL);
+			 &err, &erroffset, 0);
     
-      if (re == NULL) {
+      if (re == 0) {
 	error("%s: %s at position %d of expression", nm.c_str(), 
 	      err, erroffset);
 	return 0;
@@ -273,10 +365,10 @@ octregexp_list (const octave_value_list &args, const std::string &nm,
       char *nametable;
       int idx = 0;
 
-      pcre_fullinfo(re, NULL, PCRE_INFO_CAPTURECOUNT,  &subpatterns);
-      pcre_fullinfo(re, NULL, PCRE_INFO_NAMECOUNT, &namecount);
-      pcre_fullinfo(re, NULL, PCRE_INFO_NAMEENTRYSIZE, &nameentrysize);
-      pcre_fullinfo(re, NULL, PCRE_INFO_NAMETABLE, &nametable);
+      pcre_fullinfo(re, 0, PCRE_INFO_CAPTURECOUNT,  &subpatterns);
+      pcre_fullinfo(re, 0, PCRE_INFO_NAMECOUNT, &namecount);
+      pcre_fullinfo(re, 0, PCRE_INFO_NAMEENTRYSIZE, &nameentrysize);
+      pcre_fullinfo(re, 0, PCRE_INFO_NAMETABLE, &nametable);
 
       OCTAVE_LOCAL_BUFFER(int, ovector, (subpatterns+1)*3);
       OCTAVE_LOCAL_BUFFER(int, nidx, namecount);
@@ -293,21 +385,47 @@ octregexp_list (const octave_value_list &args, const std::string &nm,
 	{
 	  OCTAVE_QUIT;
 
-	  int matches = pcre_exec(re, NULL, buffer.c_str(), 
+	  int matches = pcre_exec(re, 0, buffer.c_str(), 
 				  buffer.length(), idx, 
 				  (idx ? PCRE_NOTBOL : 0),
 				  ovector, (subpatterns+1)*3);
 
+	  if (matches == PCRE_ERROR_MATCHLIMIT)
+	    {
+	      // try harder; start with default value for MATCH_LIMIT and increase it
+	      warning("Your pattern caused PCRE to hit its MATCH_LIMIT.\nTrying harder now, but this will be slow.");
+	      pcre_extra pe;
+	      pcre_config(PCRE_CONFIG_MATCH_LIMIT, static_cast <void *> (&pe.match_limit));
+	      pe.flags = PCRE_EXTRA_MATCH_LIMIT;
+
+	      int i = 0;
+	      while (matches == PCRE_ERROR_MATCHLIMIT &&
+		     i++ < PCRE_MATCHLIMIT_MAX)
+		{
+		  OCTAVE_QUIT;
+
+		  pe.match_limit *= 10;
+		  matches = pcre_exec(re, &pe, buffer.c_str(), 
+				      buffer.length(), idx, 
+				      (idx ? PCRE_NOTBOL : 0),
+				      ovector, (subpatterns+1)*3);
+		}
+	    }
+
 	  if (matches < 0 && matches != PCRE_ERROR_NOMATCH)
 	    {
-	      error ("%s: internal error calling pcre_exec", nm.c_str());
+	      error ("%s: internal error calling pcre_exec\nError code from pcre_exec is %i", nm.c_str(), matches);
 	      pcre_free(re);
 	      return 0;
 	    }
 	  else if (matches == PCRE_ERROR_NOMATCH)
 	    break;
 	  else if (ovector[1] <= ovector[0])
-	    break;
+	    {
+	      // FIXME: Zero sized match!! Is this the right thing to do?
+	      idx = ovector[0] + 1;
+	      continue;
+	    }
 	  else
 	    {
 	      int pos_match = 0;
@@ -375,7 +493,7 @@ octregexp_list (const octave_value_list &args, const std::string &nm,
 		      (case_insensitive ? REG_ICASE : 0));
       if (err)
 	{
-	  int len = regerror(err, &compiled, NULL, 0);
+	  int len = regerror(err, &compiled, 0, 0);
 	  OCTAVE_LOCAL_BUFFER (char, errmsg, len);
 	  regerror(err, &compiled, errmsg, len);
 	  error("%s: %s in pattern (%s)", nm.c_str(), errmsg, 
@@ -402,6 +520,9 @@ octregexp_list (const octave_value_list &args, const std::string &nm,
 	      while (matches < subexpr && match[matches].rm_so >= 0) 
 		matches++;
 
+	      if (matches == 0 || match[0].rm_eo == 0)
+		break;
+
 	      s = double (match[0].rm_so+1+idx);
 	      e = double (match[0].rm_eo+idx);
 	      Matrix te(matches-1,2);
@@ -755,7 +876,7 @@ DEFUN_DLD (regexp, args, nargout,
 @deftypefn {Loadable Function} {[@var{s}, @var{e}, @var{te}, @var{m}, @var{t}, @var{nm}] =} regexp (@var{str}, @var{pat})\n\
 @deftypefnx {Loadable Function} {[@dots{}] =} regexp (@var{str}, @var{pat}, @var{opts}, @dots{})\n\
 \n\
-Regular expression string matching. Matches @var{pat} in @var{str} and\n\
+Regular expression string matching.  Matches @var{pat} in @var{str} and\n\
 returns the position and matching substrings or empty values if there are\n\
 none.\n\
 \n\
@@ -784,16 +905,16 @@ List operators, where for example @code{[ab]c} matches @code{ac} and @code{bc}\n
 @item ()\n\
 Grouping operator\n\
 @item |\n\
-Alternation operator. Match one of a choice of regular expressions. The\n\
+Alternation operator.  Match one of a choice of regular expressions.  The\n\
 alternatives must be delimited by the grouping operator @code{()} above\n\
 @item ^ $\n\
-Anchoring operator. @code{^} matches the start of the string @var{str} and\n\
+Anchoring operator.  @code{^} matches the start of the string @var{str} and\n\
 @code{$} the end\n\
 @end table\n\
 \n\
-In addition the following escaped characters have special meaning. It should\n\
+In addition the following escaped characters have special meaning.  It should\n\
 be noted that it is recommended to quote @var{pat} in single quotes rather\n\
-than double quotes, to avoid the escape sequences being interpreted by octave\n\
+than double quotes, to avoid the escape sequences being interpreted by Octave\n\
 before being passed to @code{regexp}.\n\
 \n\
 @table @code\n\
@@ -840,12 +961,12 @@ A cell array of the text of each token matched.\n\
 \n\
 @item @var{nm}\n\
 A structure containing the text of each matched named token, with the name\n\
-being used as the fieldname. A named token is denoted as\n\
+being used as the fieldname.  A named token is denoted as\n\
 @code{(?<name>@dots{})}\n\
 @end table\n\
 \n\
 Particular output arguments or the order of the output arguments can be\n\
-selected by additional @var{opts} arguments. These are strings and the\n\
+selected by additional @var{opts} arguments.  These are strings and the\n\
 correspondence between the output arguments and the optional argument\n\
 are\n\
 \n\
@@ -859,7 +980,7 @@ are\n\
 @end multitable\n\
 \n\
 A further optional argument is 'once', that limits the number of returned\n\
-matches to the first match. Additional arguments are\n\
+matches to the first match.  Additional arguments are\n\
 \n\
 @table @asis\n\
 @item matchcase\n\
@@ -880,6 +1001,7 @@ The pattern can include arbitrary whitespace and comments starting with\n\
 @item literalspacing\n\
 The pattern is taken literally.\n\
 @end table\n\
+@seealso{regexpi, regexprep}\n\
 @end deftypefn")
 {
   octave_value_list retval;
@@ -897,6 +1019,17 @@ The pattern is taken literally.\n\
 
 /*
 
+## PCRE_ERROR_MATCHLIMIT test
+%!test
+%! s=sprintf('\t4\n0000\t-0.00\t-0.0000\t4\t-0.00\t-0.0000\t4\n0000\t-0.00\t-0.0000\t0\t-0.00\t-');
+%! ws = warning("query");
+%! unwind_protect
+%!   warning("off");
+%!   regexp(s, '(\s*-*\d+[.]*\d*\s*)+\n');
+%! unwind_protect_cleanup
+%!   warning(ws);
+%! end_unwind_protect
+
 ## seg-fault test
 %!assert(regexp("abcde","."),[1,2,3,4,5])
 
@@ -1042,6 +1175,16 @@ The pattern is taken literally.\n\
 %!assert(regexp({'asdfg-dfd';'-dfd-dfd-';'qasfdfdaq'},{'-';'f';'q'}),{6;[3,7];[1,9]})
 %!assert(regexp('Strings',{'t','s'}),{2,7})
 
+## Test case for lookaround operators
+%!assert(regexp('Iraq','q(?!u)'),4)
+%!assert(regexp('quit','q(?!u)'), zeros(1,0))
+%!assert(regexp('quit','q(?=u)','match'), {'q'})
+%!assert(regexp("quit",'q(?=u+)','match'), {'q'})
+%!assert(regexp("qit",'q(?=u+)','match'), cell(1,0))
+%!assert(regexp("qit",'q(?=u*)','match'), {'q'})
+
+%!assert(regexp('thingamabob','(?<=a)b'), 9)
+
 */
 
 DEFUN_DLD (regexpi, args, nargout,
@@ -1049,9 +1192,9 @@ DEFUN_DLD (regexpi, args, nargout,
 @deftypefn {Loadable Function} {[@var{s}, @var{e}, @var{te}, @var{m}, @var{t}, @var{nm}] =} regexpi (@var{str}, @var{pat})\n\
 @deftypefnx {Loadable Function} {[@dots{}] =} regexpi (@var{str}, @var{pat}, @var{opts}, @dots{})\n\
 \n\
-Case insensitive regular expression string matching. Matches @var{pat} in\n\
+Case insensitive regular expression string matching.  Matches @var{pat} in\n\
 @var{str} and returns the position and matching substrings or empty values\n\
-if there are none. See @code{regexp} for more details\n\
+if there are none.  @xref{doc-regexp,,regexp}, for more details\n\
 @end deftypefn")
 {
   octave_value_list retval;
@@ -1402,9 +1545,11 @@ Replace matches of @var{pat} in  @var{string} with @var{repstr}.\n\
 The replacement can contain @code{$i}, which substitutes\n\
 for the ith set of parentheses in the match string.  E.g.,\n\
 @example\n\
+@group\n\
 \n\
    regexprep(\"Bill Dunn\",'(\\w+) (\\w+)','$2, $1')\n\
 \n\
+@end group\n\
 @end example\n\
 returns \"Dunn, Bill\"\n\
 \n\
@@ -1434,7 +1579,7 @@ Whether whitespace and # comments can be used to make the regular expression mor
 Alternatively, use (?x) or (?-x) in the pattern.\n\
 \n\
 @end table\n\
-@seealso{regexp,regexpi}\n\
+@seealso{regexp,regexpi,strrep}\n\
 @end deftypefn")
 {
   octave_value_list retval;
@@ -1569,6 +1714,9 @@ Alternatively, use (?x) or (?-x) in the pattern.\n\
 %!assert(regexprep({"abc","cba"},"b","?"),{"a?c","c?a"})
 %!assert(regexprep({"abc","cba"},{"b","a"},{"?","!"}),{"!?c","c?!"})
 
+# Nasty lookbehind expression
+%!assert(regexprep('x^(-1)+y(-1)+z(-1)=0','(?<=[a-z]+)\(\-[1-9]*\)','_minus1'),'x^(-1)+y_minus1+z_minus1=0')
+
 */
 
 /*
