@@ -1,7 +1,7 @@
 // urlwrite and urlread, a curl front-end for octave
 /*
 
-Copyright (C) 2006, 2007 Alexander Barth
+Copyright (C) 2006, 2007, 2008 Alexander Barth
 
 This file is part of Octave.
 
@@ -32,6 +32,8 @@ along with Octave; see the file COPYING.  If not, see
 #include <fstream>
 #include <iomanip>
 
+#include "file-ops.h"
+#include "file-stat.h"
 #include "oct-env.h"
 
 #include "defun-dld.h"
@@ -39,6 +41,7 @@ along with Octave; see the file COPYING.  If not, see
 #include "oct-obj.h"
 #include "ov-cell.h"
 #include "pager.h"
+#include "unwind-prot.h"
 
 #if defined (HAVE_CURL)
 
@@ -167,7 +170,7 @@ urlget (const std::string& url, const std::string& method,
 
   res = CURLE_ABORTED_BY_CALLBACK;
   urlget_cleanup (curl);
-  octave_throw_interrupt_exception ();
+  octave_rethrow_exception ();
 
   BEGIN_INTERRUPT_IMMEDIATELY_IN_FOREIGN_CODE_2;
 
@@ -185,6 +188,17 @@ urlget (const std::string& url, const std::string& method,
 
 #endif
 
+static bool urlwrite_delete_file;
+
+static std::string urlwrite_filename;
+
+static void
+urlwrite_cleanup_file (void *)
+{
+  if (urlwrite_delete_file)
+    file_ops::unlink (urlwrite_filename);
+}
+
 DEFUN_DLD (urlwrite, args, nargout,
   "-*- texinfo -*-\n\
 @deftypefn {Loadable Function} {} urlwrite (@var{URL}, @var{localfile})\n\
@@ -195,8 +209,10 @@ Download a remote file specified by its @var{URL} and save it as\n\
 @var{localfile}.  For example,\n\
 \n\
 @example\n\
+@group\n\
 urlwrite (\"ftp://ftp.octave.org/pub/octave/README\", \n\
           \"README.txt\");\n\
+@end group\n\
 @end example\n\
 \n\
 The full path of the downloaded file is returned in @var{f}.  The\n\
@@ -210,8 +226,10 @@ FTP and FILE protocols.  Username and password may be specified in\n\
 the URL, for example:\n\
 \n\
 @example\n\
+@group\n\
 urlwrite (\"http://username:password@@example.com/file.txt\",\n\
           \"file.txt\");\n\
+@end group\n\
 @end example\n\
 \n\
 GET and POST requests can be specified by @var{method} and @var{param}.\n\
@@ -220,8 +238,10 @@ and @var{param} is a cell array of parameter and value pairs.\n\
 For example:\n\
 \n\
 @example\n\
+@group\n\
 urlwrite (\"http://www.google.com/search\", \"search.html\",\n\
           \"get\", @{\"query\", \"octave\"@});\n\
+@end group\n\
 @end example\n\
 @seealso{urlread}\n\
 @end deftypefn")
@@ -250,6 +270,8 @@ urlwrite (\"http://www.google.com/search\", \"search.html\",\n\
   // name to store the file if download is succesful
   std::string filename = args(1).string_value();
 
+  urlwrite_filename = filename;
+
   if (error_state)
     {
       error ("urlwrite: localfile must be a character string");
@@ -291,6 +313,14 @@ urlwrite (\"http://www.google.com/search\", \"search.html\",\n\
 	}
     }
 
+  // The file should only be deleted if it doesn't initially exist, we
+  // create it, and the download fails.  We use unwind_protect to do
+  // it so that the deletion happens no matter how we exit the function.
+
+  file_stat fs (filename);
+
+  urlwrite_delete_file = ! fs.exists ();
+
   std::ofstream ofile (filename.c_str(), std::ios::out | std::ios::binary);
 
   if (! ofile.is_open ())
@@ -299,15 +329,30 @@ urlwrite (\"http://www.google.com/search\", \"search.html\",\n\
       return retval;
     }
 
+  unwind_protect::add (urlwrite_cleanup_file);
+
   CURLcode res = urlget (url, method, param, ofile);
 
   ofile.close ();
 
+  urlwrite_delete_file = (res != CURLE_OK);
+
+  unwind_protect::run ();
+
   if (nargout > 0)
     {
-      retval(0) = octave_env::make_absolute (filename, octave_env::getcwd ());
-      retval(1) = res == CURLE_OK;
-      retval(2) = std::string (res == CURLE_OK ? "" : curl_easy_strerror (res));
+      if (res == CURLE_OK)
+	{
+	  retval(2) = std::string ();
+	  retval(1) = true;
+	  retval(0) = octave_env::make_absolute (filename, octave_env::getcwd ());
+	}
+      else
+	{
+	  retval(2) = std::string (curl_easy_strerror (res));
+	  retval(1) = false;
+	  retval(0) = std::string ();
+	}
     }
 
   if (nargout < 2 && res != CURLE_OK)
@@ -352,8 +397,10 @@ and @var{param} is a cell array of parameter and value pairs.\n\
 For example,\n\
 \n\
 @example\n\
+@group\n\
 s = urlread (\"http://www.google.com/search\", \"get\",\n\
              @{\"query\", \"octave\"@});\n\
+@end group\n\
 @end example\n\
 @seealso{urlwrite}\n\
 @end deftypefn")
