@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2004, 2005, 2006, 2007 John W. Eaton
+Copyright (C) 2004, 2005, 2006, 2007, 2008 John W. Eaton
 
 This file is part of Octave.
 
@@ -22,12 +22,11 @@ along with Octave; see the file COPYING.  If not, see
 
 #include <cstdlib>
 
-#include <iostream>
+#include <iosfwd>
 #include <string>
 
 #include "mx-base.h"
 #include "oct-alloc.h"
-#include "so-array.h"
 #include "str-vec.h"
 
 #include "error.h"
@@ -37,21 +36,25 @@ along with Octave; see the file COPYING.  If not, see
 #include "ov-typeinfo.h"
 #include "gripes.h"
 
+#include "ov-re-mat.h"
+#include "ov-scalar.h"
+
 class
 OCTINTERP_API
 OCTAVE_VALUE_INT_MATRIX_T
-  : public octave_base_int_matrix<OCTAVE_INT_NDARRAY_T>
+  : public octave_base_int_matrix<intNDArray<OCTAVE_INT_T> >
 {
 public:
 
   OCTAVE_VALUE_INT_MATRIX_T (void)
-    : octave_base_int_matrix<OCTAVE_INT_NDARRAY_T> () { }
+    : octave_base_int_matrix<intNDArray<OCTAVE_INT_T> > () { }
 
-  OCTAVE_VALUE_INT_MATRIX_T (const OCTAVE_INT_NDARRAY_T& nda)
-    : octave_base_int_matrix<OCTAVE_INT_NDARRAY_T> (nda) { }
+  OCTAVE_VALUE_INT_MATRIX_T (const intNDArray<OCTAVE_INT_T>& nda)
+    : octave_base_int_matrix<intNDArray<OCTAVE_INT_T> > (nda) { }
 
   OCTAVE_VALUE_INT_MATRIX_T (const ArrayN<OCTAVE_INT_T>& nda)
-    : octave_base_int_matrix<OCTAVE_INT_NDARRAY_T> (OCTAVE_INT_NDARRAY_T (nda)) { }
+    : octave_base_int_matrix<intNDArray<OCTAVE_INT_T> >
+        (intNDArray<OCTAVE_INT_T> (nda)) { }
 
   ~OCTAVE_VALUE_INT_MATRIX_T (void) { }
 
@@ -65,29 +68,47 @@ public:
 
   bool is_integer_type (void) const { return true; }
 
+private:
+
+  template <class IM>
+  IM convert_gripe () const
+    {
+      typedef typename IM::element_type dest_el_type;
+      typedef intNDArray<OCTAVE_INT_T>::element_type src_el_type;
+      dest_el_type::clear_conv_flag ();
+      IM retval (matrix);
+      if (dest_el_type::get_trunc_flag ())
+	gripe_truncated_conversion (src_el_type::type_name (),
+				    dest_el_type::type_name ());
+      dest_el_type::clear_conv_flag ();
+      return retval;
+    }
+
+public:
+
   int8NDArray
-  int8_array_value (void) const { return int8NDArray (matrix); }
+  int8_array_value (void) const { return convert_gripe<int8NDArray> (); }
 
   int16NDArray
-  int16_array_value (void) const { return int16NDArray (matrix); }
+  int16_array_value (void) const { return convert_gripe<int16NDArray> (); }
 
   int32NDArray
-  int32_array_value (void) const { return int32NDArray (matrix); }
+  int32_array_value (void) const { return convert_gripe<int32NDArray> (); }
 
   int64NDArray
-  int64_array_value (void) const { return int64NDArray (matrix); }
+  int64_array_value (void) const { return convert_gripe<int64NDArray> (); }
 
   uint8NDArray
-  uint8_array_value (void) const { return uint8NDArray (matrix); }
+  uint8_array_value (void) const { return convert_gripe<uint8NDArray> (); }
 
   uint16NDArray
-  uint16_array_value (void) const { return uint16NDArray (matrix); }
+  uint16_array_value (void) const { return convert_gripe<uint16NDArray> (); }
 
   uint32NDArray
-  uint32_array_value (void) const { return uint32NDArray (matrix); }
+  uint32_array_value (void) const { return convert_gripe<uint32NDArray> (); }
 
   uint64NDArray
-  uint64_array_value (void) const { return uint64NDArray (matrix); }
+  uint64_array_value (void) const { return convert_gripe<uint64NDArray> (); }
 
   double
   double_value (bool = false) const
@@ -108,8 +129,29 @@ public:
       
     }
 
+  float
+  float_value (bool = false) const
+    {
+      float retval = lo_ieee_float_nan_value ();
+
+      if (numel () > 0)
+	{
+	  gripe_implicit_conversion ("Octave:array-as-scalar",
+				     type_name (), "real scalar");
+
+	  retval = matrix(0).float_value ();
+	}
+      else
+	gripe_invalid_conversion (type_name (), "real scalar");
+
+      return retval;
+      
+    }
+
   double scalar_value (bool = false) const { return double_value (); }
 
+  float float_scalar_value (bool = false) const { return float_value (); }
+
   Matrix
   matrix_value (bool = false) const
     {
@@ -128,6 +170,24 @@ public:
       return retval;
     }
 
+  FloatMatrix
+  float_matrix_value (bool = false) const
+    {
+      FloatMatrix retval;
+      dim_vector dv = dims ();
+      if (dv.length () > 2)
+	error ("invalid conversion of %s to FloatMatrix", type_name().c_str ());
+      else
+	{
+	  retval = FloatMatrix (dv(0), dv(1));
+	  float *vec = retval.fortran_vec ();
+	  octave_idx_type nel = matrix.numel ();
+	  for (octave_idx_type i = 0; i < nel; i++)
+	    vec[i] = matrix(i).float_value ();
+	}
+      return retval;
+    }
+
   ComplexMatrix
   complex_matrix_value (bool = false) const
     {
@@ -146,6 +206,24 @@ public:
       return retval;
     }
 
+  FloatComplexMatrix
+  float_complex_matrix_value (bool = false) const
+    {
+      FloatComplexMatrix retval;
+      dim_vector dv = dims();
+      if (dv.length () > 2)
+	error ("invalid conversion of %s to FloatMatrix", type_name().c_str ());
+      else
+	{
+	  retval = FloatComplexMatrix (dv(0), dv(1));
+	  FloatComplex *vec = retval.fortran_vec ();
+	  octave_idx_type nel = matrix.numel ();
+	  for (octave_idx_type i = 0; i < nel; i++)
+	    vec[i] = FloatComplex (matrix(i).float_value ());
+	}
+      return retval;
+    }
+
   NDArray
   array_value (bool = false) const
     { 
@@ -157,6 +235,17 @@ public:
       return retval;
     }
 
+  FloatNDArray
+  float_array_value (bool = false) const
+    { 
+      FloatNDArray retval (matrix.dims ()); 
+      float *vec = retval.fortran_vec ();
+      octave_idx_type nel = matrix.numel ();
+      for (octave_idx_type i = 0; i < nel; i++)
+        vec[i] = matrix(i).float_value ();
+      return retval;
+    }
+
   ComplexNDArray
   complex_array_value (bool = false) const
     { 
@@ -168,6 +257,17 @@ public:
       return retval;
     }
 
+  FloatComplexNDArray
+  float_complex_array_value (bool = false) const
+    { 
+      FloatComplexNDArray retval (matrix.dims ()); 
+      FloatComplex *vec = retval.fortran_vec ();
+      octave_idx_type nel = matrix.numel ();
+      for (octave_idx_type i = 0; i < nel; i++)
+        vec[i] = FloatComplex (matrix(i).float_value ());
+      return retval;
+    }
+
   boolNDArray
   bool_array_value (bool warn = false) const
   {
@@ -199,9 +299,22 @@ public:
     return retval;
   }
 
-  void increment (void) { matrix += 1; }
+  void increment (void) 
+   { 
+     matrix += OCTAVE_INT_T (1); 
+     if (OCTAVE_INT_T::get_math_trunc_flag ())
+       gripe_unop_integer_math_truncated ("++", type_name (). c_str ());
 
-  void decrement (void) { matrix -= 1; }
+      OCTAVE_INT_T::clear_conv_flag ();
+   }
+
+  void decrement (void)
+   { 
+     matrix -= OCTAVE_INT_T (1); 
+     if (OCTAVE_INT_T::get_math_trunc_flag ())
+       gripe_unop_integer_math_truncated ("--", type_name (). c_str ());
+      OCTAVE_INT_T::clear_conv_flag ();
+   }
 
   idx_vector index_vector (void) const { return idx_vector (matrix); }
 
@@ -210,6 +323,10 @@ public:
 	     oct_mach_info::float_format flt_fmt) const
     { return os.write (matrix, block_size, output_type, skip, flt_fmt); }
 
+  // Unsafe.  This function exists to support the MEX interface.
+  // You should not use it anywhere else.
+  void *mex_get_data (void) const { return matrix.mex_get_data (); }
+
   mxArray *as_mxArray (void) const
   {
     mxArray *retval = new mxArray (OCTAVE_INT_MX_CLASS, dims (), mxREAL);
@@ -226,6 +343,43 @@ public:
     return retval;
   }
 
+#define MAT_MAPPER(FCN) \
+  octave_value FCN (void) const { return matrix.FCN (); }
+
+  MAT_MAPPER (abs)
+  MAT_MAPPER (signum)
+
+#undef MAT_MAPPER
+
+  octave_value imag (void) const
+  {
+    return intNDArray<OCTAVE_INT_T> (matrix.dims (),
+				     static_cast<OCTAVE_INT_T>(0));
+  }
+
+#define NO_OP_MAPPER(FCN) \
+  octave_value FCN (void) const { return octave_value (matrix); }
+
+  NO_OP_MAPPER (ceil)
+  NO_OP_MAPPER (conj)
+  NO_OP_MAPPER (fix)
+  NO_OP_MAPPER (floor)
+  NO_OP_MAPPER (real)
+  NO_OP_MAPPER (round)
+  NO_OP_MAPPER (roundb)
+
+#undef NO_OP_MAPPER
+
+#define BOOL_MAPPER(FCN, VAL) \
+  octave_value FCN (void) const { return boolNDArray (matrix.dims (), VAL); }
+
+  BOOL_MAPPER (finite, true)
+  BOOL_MAPPER (isinf, false)
+  BOOL_MAPPER (isna, false)
+  BOOL_MAPPER (isnan, false)
+
+#undef BOOL_MAPPER
+
 private:
 
   DECLARE_OCTAVE_ALLOCATOR
@@ -252,109 +406,119 @@ public:
     { return new OCTAVE_VALUE_INT_SCALAR_T (*this); }
 
   octave_base_value *empty_clone (void) const
-    { return new OCTAVE_VALUE_INT_SCALAR_T (); }
+    { return new OCTAVE_VALUE_INT_MATRIX_T (); }
 
   octave_value do_index_op (const octave_value_list& idx,
 			    bool resize_ok = false)
     {
-      octave_value retval;
+      // FIXME -- this doesn't solve the problem of
+      //
+      //   a = 1; a([1,1], [1,1], [1,1])
+      //
+      // and similar constructions.  Hmm...
 
-      if (idx.valid_scalar_indices ())
-	retval = scalar;
-      else
-	{
-	  // FIXME -- this doesn't solve the problem of
-	  //
-	  //   a = 1; a([1,1], [1,1], [1,1])
-	  //
-	  // and similar constructions.  Hmm...
+      // FIXME -- using this constructor avoids narrowing the
+      // 1x1 matrix back to a scalar value.  Need a better solution
+      // to this problem.
 
-	  // FIXME -- using this constructor avoids narrowing the
-	  // 1x1 matrix back to a scalar value.  Need a better solution
-	  // to this problem.
+      octave_value tmp
+        (new OCTAVE_VALUE_INT_MATRIX_T
+         (OCTAVE_VALUE_INT_NDARRAY_EXTRACTOR_FUNCTION ())); 
 
-	  octave_value tmp
-	    (new OCTAVE_VALUE_INT_MATRIX_T
-	     (OCTAVE_VALUE_INT_NDARRAY_EXTRACTOR_FUNCTION ())); 
-
-	  retval = tmp.do_index_op (idx, resize_ok);
-	}
-
-      return retval;
+      return tmp.do_index_op (idx, resize_ok);
     }
 
   bool OCTAVE_TYPE_PREDICATE_FUNCTION (void) const { return true; }
 
   bool is_integer_type (void) const { return true; }
 
+private:
+
+  template <class IS>
+  IS convert_gripe () const
+    {
+      typedef IS dest_el_type;
+      typedef OCTAVE_INT_T src_el_type;
+      dest_el_type::clear_conv_flag ();
+      IS retval (scalar);
+
+      if (dest_el_type::get_trunc_flag ())
+	gripe_truncated_conversion (src_el_type::type_name (),
+				    dest_el_type::type_name ());
+      dest_el_type::clear_conv_flag ();
+      return retval;
+    }
+
+public:
+
   octave_int8
-  int8_scalar_value (void) const { return octave_int8 (scalar); }
+  int8_scalar_value (void) const { return convert_gripe<octave_int8> (); }
 
   octave_int16
-  int16_scalar_value (void) const { return octave_int16 (scalar); }
+  int16_scalar_value (void) const { return convert_gripe<octave_int16> (); }
 
   octave_int32
-  int32_scalar_value (void) const { return octave_int32 (scalar); }
+  int32_scalar_value (void) const { return convert_gripe<octave_int32> (); }
 
   octave_int64
-  int64_scalar_value (void) const { return octave_int64 (scalar); }
+  int64_scalar_value (void) const { return convert_gripe<octave_int64> (); }
 
   octave_uint8
-  uint8_scalar_value (void) const { return octave_uint8 (scalar); }
+  uint8_scalar_value (void) const { return convert_gripe<octave_uint8> (); }
 
   octave_uint16
-  uint16_scalar_value (void) const { return octave_uint16 (scalar); }
+  uint16_scalar_value (void) const { return convert_gripe<octave_uint16> (); }
 
   octave_uint32
-  uint32_scalar_value (void) const { return octave_uint32 (scalar); }
+  uint32_scalar_value (void) const { return convert_gripe<octave_uint32> (); }
 
   octave_uint64
-  uint64_scalar_value (void) const { return octave_uint64 (scalar); }
+  uint64_scalar_value (void) const { return convert_gripe<octave_uint64> (); }
 
   int8NDArray
   int8_array_value (void) const
-    { return int8NDArray (dim_vector (1, 1), scalar); }
+    { return int8NDArray (dim_vector (1, 1), int8_scalar_value ()); }
 
   int16NDArray
   int16_array_value (void) const
-    { return int16NDArray (dim_vector (1, 1), scalar); }
+    { return int16NDArray (dim_vector (1, 1), int16_scalar_value ()); }
 
   int32NDArray
   int32_array_value (void) const
-    { return int32NDArray (dim_vector (1, 1), scalar); }
+    { return int32NDArray (dim_vector (1, 1), int32_scalar_value ()); }
 
   int64NDArray
   int64_array_value (void) const
-    { return int64NDArray (dim_vector (1, 1), scalar); }
+    { return int64NDArray (dim_vector (1, 1), int64_scalar_value ()); }
 
   uint8NDArray
   uint8_array_value (void) const
-    { return uint8NDArray (dim_vector (1, 1), scalar); }
+    { return uint8NDArray (dim_vector (1, 1), uint8_scalar_value ()); }
 
   uint16NDArray
   uint16_array_value (void) const
-    { return uint16NDArray (dim_vector (1, 1), scalar); }
+    { return uint16NDArray (dim_vector (1, 1), uint16_scalar_value ()); }
 
   uint32NDArray
   uint32_array_value (void) const
-    { return uint32NDArray (dim_vector (1, 1), scalar); }
+    { return uint32NDArray (dim_vector (1, 1), uint32_scalar_value ()); }
 
   uint64NDArray
   uint64_array_value (void) const
-    { return uint64NDArray (dim_vector (1, 1), scalar); }
+    { return uint64NDArray (dim_vector (1, 1), uint64_scalar_value ()); }
 
   octave_value resize (const dim_vector& dv, bool fill = false) const
     {
       if (fill)
 	{
-	  OCTAVE_INT_NDARRAY_T retval (dv, 0);
+	  intNDArray<OCTAVE_INT_T> retval (dv, 0);
 	  if (dv.numel())
 	    retval(0) = scalar;
 	  return retval;
 	}
       else
 	{
-	  OCTAVE_INT_NDARRAY_T retval (dv);
+	  intNDArray<OCTAVE_INT_T> retval (dv);
 	  if (dv.numel())
 	    retval(0) = scalar;
 	  return retval;
@@ -363,8 +527,12 @@ public:
 
   double double_value (bool = false) const { return scalar.double_value (); }
 
+  float float_value (bool = false) const { return scalar.float_value (); }
+
   double scalar_value (bool = false) const { return scalar.double_value (); }
 
+  float float_scalar_value (bool = false) const { return scalar.float_value (); }
+
   Matrix
   matrix_value (bool = false) const
     {
@@ -373,6 +541,14 @@ public:
       return retval;
     }
 
+  FloatMatrix
+  float_matrix_value (bool = false) const
+    {
+      FloatMatrix retval (1, 1);
+      retval(0,0) = scalar.float_value ();
+      return retval;
+    }
+
   ComplexMatrix
   complex_matrix_value (bool = false) const
     {
@@ -381,6 +557,13 @@ public:
       return retval;
     }
 
+  FloatComplexMatrix
+  float_complex_matrix_value (bool = false) const
+    {
+      FloatComplexMatrix retval (1, 1);
+      retval(0,0) = FloatComplex (scalar.float_value ());
+      return retval;
+    }
 
   NDArray
   array_value (bool = false) const
@@ -390,11 +573,27 @@ public:
       return retval;
     }
 
+  FloatNDArray
+  float_array_value (bool = false) const
+    { 
+      FloatNDArray retval (dim_vector (1, 1)); 
+      retval(0) = scalar.float_value ();
+      return retval;
+    }
+
   ComplexNDArray
   complex_array_value (bool = false) const
     { 
       ComplexNDArray retval (dim_vector (1, 1));
-      retval(0) = Complex (scalar.double_value ());
+      retval(0) = FloatComplex (scalar.double_value ());
+      return retval;
+    }
+
+  FloatComplexNDArray
+  float_complex_array_value (bool = false) const
+    { 
+      FloatComplexNDArray retval (dim_vector (1, 1));
+      retval(0) = FloatComplex (scalar.float_value ());
       return retval;
     }
 
@@ -419,9 +618,21 @@ public:
     return retval;
   }
 
-  void increment (void) { scalar += 1; }
+  void increment (void) 
+   { 
+     scalar += OCTAVE_INT_T (1); 
+     if (OCTAVE_INT_T::get_math_trunc_flag ())
+       gripe_unop_integer_math_truncated ("++", type_name (). c_str ());
+      OCTAVE_INT_T::clear_conv_flag ();
+   }
 
-  void decrement (void) { scalar -= 1; }
+  void decrement (void)
+   { 
+     scalar -= OCTAVE_INT_T (1); 
+     if (OCTAVE_INT_T::get_math_trunc_flag ())
+       gripe_unop_integer_math_truncated ("--", type_name (). c_str ());
+      OCTAVE_INT_T::clear_conv_flag ();
+   }
 
   idx_vector index_vector (void) const { return idx_vector (scalar); }
 
@@ -448,6 +659,38 @@ public:
     return retval;
   }
 
+#define SCALAR_MAPPER(FCN) \
+  octave_value FCN (void) const { return scalar.FCN (); }
+
+  SCALAR_MAPPER (abs)
+  SCALAR_MAPPER (signum)
+
+#undef SCALAR_MAPPER
+
+  octave_value imag (void) const { return static_cast<OCTAVE_INT_T>(0); }
+
+#define NO_OP_MAPPER(FCN) \
+  octave_value FCN (void) const { return octave_value (scalar); }
+
+  NO_OP_MAPPER (ceil)
+  NO_OP_MAPPER (conj)
+  NO_OP_MAPPER (fix)
+  NO_OP_MAPPER (floor)
+  NO_OP_MAPPER (real)
+  NO_OP_MAPPER (round)
+  NO_OP_MAPPER (roundb)
+
+#undef NO_OP_MAPPER
+
+#define BOOL_MAPPER(FCN, VAL) octave_value FCN (void) const { return VAL; }
+
+  BOOL_MAPPER (finite, true)
+  BOOL_MAPPER (isinf, false)
+  BOOL_MAPPER (isna, false)
+  BOOL_MAPPER (isnan, false)
+
+#undef BOOL_MAPPER
+
 private:
 
   DECLARE_OCTAVE_ALLOCATOR
