@@ -876,7 +876,7 @@ SPARSE_MAPPER (erfc, SparseMatrix::dmapper, ::erfc)
 SPARSE_MAPPER (gamma, SparseMatrix::dmapper, xgamma)
 CD_SPARSE_MAPPER (lgamma, xlgamma, xlgamma, 0.0, octave_Inf)
 SPARSE_MAPPER (abs, SparseMatrix::dmapper, ::fabs)
-SPARSE_MAPPER (acos, SparseMatrix::dmapper, ::acos)
+CD_SPARSE_MAPPER (acos, ::acos, ::acos, -1.0, 1.0)
 CD_SPARSE_MAPPER (acosh, ::acosh, ::acosh, 1.0, octave_Inf)
 SPARSE_MAPPER (angle, SparseMatrix::dmapper, ::arg)
 SPARSE_MAPPER (arg, SparseMatrix::dmapper, ::arg)
