@@ -1,7 +1,7 @@
 /*
 
-Copyright (C) 1996, 1997, 2000, 2002, 2003, 2004, 2005, 2006, 2007
-              John W. Eaton
+Copyright (C) 1996, 1997, 2000, 2002, 2003, 2004, 2005, 2006, 2007,
+              2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -91,7 +91,7 @@ tree_checker::visit_continue_command (tree_continue_command&)
 }
 
 void
-tree_checker::visit_decl_command (tree_decl_command& cmd)
+tree_checker::do_decl_command (tree_decl_command& cmd)
 {
   tree_decl_init_list *init_list = cmd.initializer_list ();
 
@@ -99,6 +99,18 @@ tree_checker::visit_decl_command (tree_decl_command& cmd)
     init_list->accept (*this);
 }
 
+void
+tree_checker::visit_global_command (tree_global_command& cmd)
+{
+  do_decl_command (cmd);
+}
+
+void
+tree_checker::visit_static_command (tree_static_command& cmd)
+{
+  do_decl_command (cmd);
+}
+
 void
 tree_checker::visit_decl_elt (tree_decl_elt& cmd)
 {
@@ -180,6 +192,15 @@ tree_checker::visit_complex_for_command (tree_complex_for_command& cmd)
     list->accept (*this);
 }
 
+void
+tree_checker::visit_octave_user_script (octave_user_script& fcn)
+{
+  tree_statement_list *cmd_list = fcn.body ();
+
+  if (cmd_list)
+    cmd_list->accept (*this);
+}
+
 void
 tree_checker::visit_octave_user_function (octave_user_function& fcn)
 {
@@ -189,6 +210,17 @@ tree_checker::visit_octave_user_function (octave_user_function& fcn)
     cmd_list->accept (*this);
 }
 
+void
+tree_checker::visit_function_def (tree_function_def& fdef)
+{
+  octave_value fcn = fdef.function ();
+
+  octave_function *f = fcn.function_value ();
+
+  if (f)
+    f->accept (*this);
+}
+
 void
 tree_checker::visit_identifier (tree_identifier& /* id */)
 {
