@@ -1,7 +1,7 @@
 /*
 
-Copyright (C) 1994, 1995, 1996, 1997, 2002, 2004, 2005, 2006, 2007
-              John W. Eaton
+Copyright (C) 1994, 1995, 1996, 1997, 2002, 2004, 2005, 2006, 2007,
+              2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -31,7 +31,8 @@ along with Octave; see the file COPYING.  If not, see
 // No-op.
 
 tree_command *
-tree_no_op_command::dup (symbol_table *sym_tab)
+tree_no_op_command::dup (symbol_table::scope_id,
+			 symbol_table::context_id) const
 {
   return new tree_no_op_command (orig_cmd, line (), column ());
 }
@@ -42,6 +43,21 @@ tree_no_op_command::accept (tree_walker& tw)
   tw.visit_no_op_command (*this);
 }
 
+// Function definition.
+
+tree_command *
+tree_function_def::dup (symbol_table::scope_id,
+			symbol_table::context_id) const
+{
+  return new tree_function_def (fcn, line (), column ());
+}
+
+void
+tree_function_def::accept (tree_walker& tw)
+{
+  tw.visit_function_def (*this);
+}
+
 /*
 ;;; Local Variables: ***
 ;;; mode: C++ ***
