@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1996, 1997, 1998, 2000, 2002, 2003, 2004, 2005, 2006,
-              2007 John W. Eaton
+              2007, 2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
@@ -34,21 +34,23 @@ along with Octave; see the file COPYING.  If not, see
 #include "pager.h"
 #include "pt-arg-list.h"
 #include "pt-bp.h"
+#include "pt-id.h"
 #include "pt-idx.h"
 #include "pt-walk.h"
 #include "utils.h"
 #include "variables.h"
+#include "gripes.h"
 
 // Index expressions.
 
 tree_index_expression::tree_index_expression (int l, int c)
-  : tree_expression (l, c), expr (), args (), type (),
+  : tree_expression (l, c), expr (0), args (0), type (),
     arg_nm (), dyn_field () { }
 
 tree_index_expression::tree_index_expression (tree_expression *e,
 					      tree_argument_list *lst,
 					      int l, int c, char t)
-  : tree_expression (l, c), expr (e), args (), type (),
+  : tree_expression (l, c), expr (e), args (0), type (),
     arg_nm (), dyn_field ()
 {
   append (lst, t);
@@ -57,7 +59,7 @@ tree_index_expression::tree_index_expression (tree_expression *e,
 tree_index_expression::tree_index_expression (tree_expression *e,
 					      const std::string& n,
 					      int l, int c)
-  : tree_expression (l, c), expr (e), args (), type (),
+  : tree_expression (l, c), expr (e), args (0), type (),
     arg_nm (), dyn_field ()
 {
   append (n);
@@ -66,7 +68,7 @@ tree_index_expression::tree_index_expression (tree_expression *e,
 tree_index_expression::tree_index_expression (tree_expression *e,
 					      tree_expression *df,
 					      int l, int c)
-  : tree_expression (l, c), expr (e), args (), type (),
+  : tree_expression (l, c), expr (e), args (0), type (),
     arg_nm (), dyn_field ()
 {
   append (df);
@@ -171,7 +173,12 @@ make_value_list (tree_argument_list *args, const string_vector& arg_nm,
   octave_value_list retval;
 
   if (args)
-    retval = args->convert_to_const_vector (object);
+    {
+      if (object && args->has_magic_end () && object->is_undefined ())
+        gripe_invalid_inquiry_subscript ();
+      else
+        retval = args->convert_to_const_vector (object);
+    }
 
   if (! error_state)
     {
@@ -197,7 +204,7 @@ tree_index_expression::get_struct_index
 
       if (df)
 	{
-	  octave_value t = df->rvalue ();
+	  octave_value t = df->rvalue1 ();
 
 	  if (! error_state)
 	    {
@@ -219,9 +226,8 @@ tree_index_expression::make_arg_struct (void) const
 {
   int n = args.size ();
 
-  // FIXME -- why not just make these Cell objects?
-  octave_value_list subs_list (n, octave_value ());
-  octave_value_list type_list (n, octave_value ());
+  Cell type_field (n, 1);
+  Cell subs_field (n, 1);
 
   std::list<tree_argument_list *>::const_iterator p_args = args.begin ();
   std::list<string_vector>::const_iterator p_arg_nm = arg_nm.begin ();
@@ -234,20 +240,15 @@ tree_index_expression::make_arg_struct (void) const
       switch (type[i])
 	{
 	case '(':
-	  subs_list(i) = make_subs_cell (*p_args, *p_arg_nm);
+	  subs_field(i) = make_subs_cell (*p_args, *p_arg_nm);
 	  break;
 
 	case '{':
-	  subs_list(i) = make_subs_cell (*p_args, *p_arg_nm);
+	  subs_field(i) = make_subs_cell (*p_args, *p_arg_nm);
 	  break;
 
 	case '.':
-	  {
-	    subs_list(i) = get_struct_index (p_arg_nm, p_dyn_field);
-
-	    if (error_state)
-	      eval_error ();
-	  }
+	  subs_field(i) = get_struct_index (p_arg_nm, p_dyn_field);
 	  break;
 
 	default:
@@ -262,8 +263,8 @@ tree_index_expression::make_arg_struct (void) const
       p_dyn_field++;
     }
 
-  m.assign ("subs", Cell (subs_list));
-  m.assign ("type", Cell (type_list));
+  m.assign ("type", type_field);
+  m.assign ("subs", subs_field);
 
   return m;
 }
@@ -276,11 +277,39 @@ tree_index_expression::rvalue (int nargout)
   if (error_state)
     return retval;
 
-  octave_value first_expr_val = expr->rvalue ();
-  octave_value tmp = first_expr_val;
+  octave_value first_expr_val;
+
+  octave_value_list first_args;
+
+  bool have_args = false;
+
+  if (expr->is_identifier () && type[0] == '(')
+    {
+      tree_identifier *id = dynamic_cast<tree_identifier *> (expr);
+
+      if (! (id->is_variable () || args.empty ()))
+	{
+	  tree_argument_list *al = *(args.begin ());
+
+	  size_t n = al ? al->length () : 0;
+
+	  if (n > 0)
+	    {
+	      string_vector anm = *(arg_nm.begin ());
+
+	      first_expr_val = id->do_lookup  (al, anm, first_args, have_args);
+	    }
+	}
+    }
 
   if (! error_state)
     {
+      if (first_expr_val.is_undefined ())
+	first_expr_val = expr->rvalue1 ();
+
+      octave_value tmp = first_expr_val;
+      octave_idx_type tmpi = 0;
+
       std::list<octave_value_list> idx;
 
       int n = args.size ();
@@ -304,13 +333,18 @@ tree_index_expression::rvalue (int nargout)
 		  // and we are looking at the argument list that
 		  // contains the second (or third, etc.) "end" token,
 		  // so we must evaluate everything up to the point of
-		  // that argument list so we pass the appropiate
+		  // that argument list so we can pass the appropriate
 		  // value to the built-in __end__ function.
 
-		  octave_value_list tmp_list
-		    = first_expr_val.subsref (type, idx, nargout);
+		  const octave_value_list tmp_list
+		    = tmp.subsref (type.substr (tmpi, i - tmpi), idx, nargout);
 
 		  tmp = tmp_list(0);
+                  tmpi = i;
+                  idx.clear ();
+                  
+                  if (tmp.is_cs_list ())
+                    gripe_indexed_cs_list ();
 
 		  if (error_state)
 		    break;
@@ -320,7 +354,13 @@ tree_index_expression::rvalue (int nargout)
 	  switch (type[i])
 	    {
 	    case '(':
-	      idx.push_back (make_value_list (*p_args, *p_arg_nm, &tmp));
+	      if (have_args)
+		{
+		  idx.push_back (first_args);
+		  have_args = false;
+		}
+	      else
+		idx.push_back (make_value_list (*p_args, *p_arg_nm, &tmp));
 	      break;
 
 	    case '{':
@@ -328,12 +368,7 @@ tree_index_expression::rvalue (int nargout)
 	      break;
 
 	    case '.':
-	      {
-		idx.push_back (octave_value (get_struct_index (p_arg_nm, p_dyn_field)));
-
-		if (error_state)
-		  eval_error ();
-	      }
+	      idx.push_back (octave_value (get_struct_index (p_arg_nm, p_dyn_field)));
 	      break;
 
 	    default:
@@ -349,18 +384,18 @@ tree_index_expression::rvalue (int nargout)
 	}
 
       if (! error_state)
-	retval = first_expr_val.subsref (type, idx, nargout);
+	retval = tmp.subsref (type.substr (tmpi, n - tmpi), idx, nargout);
     }
 
   return retval;
 }
 
 octave_value
-tree_index_expression::rvalue (void)
+tree_index_expression::rvalue1 (int nargout)
 {
   octave_value retval;
 
-  octave_value_list tmp = rvalue (1);
+  const octave_value_list tmp = rvalue (nargout);
 
   if (! tmp.empty ())
     retval = tmp(0);
@@ -368,12 +403,39 @@ tree_index_expression::rvalue (void)
   return retval;
 }
 
+static octave_idx_type
+get_numel (const octave_value& val,
+           const octave_value_list& idx)
+{
+  octave_idx_type retval;
+
+  octave_idx_type len = idx.length ();
+
+  if (len == 0)
+    retval = val.numel ();
+  else
+    {
+      const dim_vector dv = val.dims ().redim (len);
+      retval = 1;
+      for (octave_idx_type i = 0; i < len; i++)
+        {
+          if (idx(i).is_magic_colon ())
+            retval *= dv(i);
+          else
+            retval *= idx(i).numel ();
+        }
+    }
+
+  return retval;
+}
+
 octave_lvalue
 tree_index_expression::lvalue (void)
 {
   octave_lvalue retval;
 
   std::list<octave_value_list> idx;
+  std::string tmp_type;
 
   int n = args.size ();
 
@@ -385,202 +447,129 @@ tree_index_expression::lvalue (void)
 
   if (! error_state)
     {
-      bool have_new_struct_field = false;
-
-      octave_idx_type new_struct_field_nel = 0;
-
-      // I think it is OK to have a copy here.
-
       const octave_value *tro = retval.object ();
 
-      octave_value first_retval_object;
+      octave_value tmp;
 
       if (tro)
-	first_retval_object = *tro;
+	tmp = *tro;
 
-      octave_value tmp = first_retval_object;
+      octave_idx_type tmpi = 0;
+      std::list<octave_value_list> tmpidx;
 
       for (int i = 0; i < n; i++)
 	{
-	  if (i > 0)
-	    {
-	      tree_argument_list *al = *p_args;
+          if (retval.numel () != 1)
+            gripe_indexed_cs_list ();
+          else if (tmpi < i)
+            {
+              tmp = tmp.subsref (type.substr (tmpi, i - tmpi), tmpidx, true);
+              tmpidx.clear ();
+            }
 
-	      if (al && al->has_magic_end ())
-		{
-		  // We have an expression like
-		  //
-		  //   x{end}.a(end)
-		  //
-		  // and we are looking at the argument list that
-		  // contains the second (or third, etc.) "end" token,
-		  // so we must evaluate everything up to the point of
-		  // that argument list so we pass the appropiate
-		  // value to the built-in __end__ function.
-
-		  octave_value_list tmp_list
-		    = first_retval_object.subsref (type, idx, 1);
-
-		  tmp = tmp_list(0);
-
-		  if (error_state)
-		    break;
-		}
-	    }
+          if (error_state)
+            break;
 
 	  switch (type[i])
 	    {
 	    case '(':
-	      idx.push_back (make_value_list (*p_args, *p_arg_nm, &tmp));
-	      break;
+              {
+                octave_value_list tidx
+                  = make_value_list (*p_args, *p_arg_nm, &tmp);
+
+                idx.push_back (tidx);
+
+                if (i < n - 1)
+                  {
+                    if (type[i+1] == '.')
+                      {
+                        tmpidx.push_back (tidx);
+                        tmpi = i+1;
+                      }
+                    else
+                      error ("() must be followed by . or close the index chain");
+                  }
+              }
+              break;
 
 	    case '{':
 	      {
 		octave_value_list tidx
 		  = make_value_list (*p_args, *p_arg_nm, &tmp);
 
+                if (tmp.is_undefined ())
+                  {
+                    if (tidx.has_magic_colon ())
+                      gripe_invalid_inquiry_subscript ();
+                    else
+                      tmp = Cell ();
+                  }
+                else if (tmp.is_zero_by_zero () 
+                         && (tmp.is_matrix_type () || tmp.is_string ()))
+                  {
+                    tmp = Cell ();
+                  }
+
+                retval.numel (get_numel (tmp, tidx));
+
+                if (error_state)
+                  break;
+
 		idx.push_back (tidx);
-
-		if (i == n-1)
-		  {
-		    // Last indexing element.  Will this result in a
-		    // comma-separated list?
-
-		    if (tidx.has_magic_colon ())
-		      {
-			octave_value_list tmp_list
-			  = first_retval_object.subsref (type, idx, 1);
-
-			if (! error_state)
-			  {
-			    octave_value val = tmp_list(0);
-
-			    if (val.is_cs_list ())
-			      retval.numel (val.numel ());
-			  }
-		      }
-		    else
-		      {
-			octave_idx_type nel = 1;
-
-			octave_idx_type nidx = tidx.length ();
-
-			for (octave_idx_type j = 0; j < nidx; j++)
-			  {
-			    octave_value val = tidx(j);
-
-			    nel *= val.numel ();
-			  }
-
-			retval.numel (nel);
-		      }
-		  }
+                tmpidx.push_back (tidx);
+                tmpi = i;
 	      }
 	      break;
 
 	    case '.':
 	      {
 		octave_value tidx = get_struct_index (p_arg_nm, p_dyn_field);
+                if (error_state)
+                  break;
 
-		if (! error_state)
-		  {
-		    if (i == n-1)
-		      {
-			// Last indexing element.  Will this result in a
-			// comma-separated list?
+                bool autoconv = (tmp.is_zero_by_zero () 
+                                 && (tmp.is_matrix_type () || tmp.is_string ()
+                                     || tmp.is_cell ()));
 
-			if (have_new_struct_field)
-			  retval.numel (new_struct_field_nel);
-			else if (i > 0)
-			  {
-			    std::string ttype = type.substr (0, i);
+                if (i > 0 && type [i-1] == '(')
+                  {
+                    octave_value_list pidx = idx.back ();
 
-			    char c = ttype[ttype.length()-1];
-			    if (c == '(' || c == '{')
-			      {
-				octave_idx_type nel = 1;
+                    if (tmp.is_undefined ())
+                      {
+                        if (pidx.has_magic_colon ())
+                          gripe_invalid_inquiry_subscript ();
+                        else
+                          tmp = Octave_map ();
+                      }
+                    else if (autoconv)
+                      tmp = Octave_map ();
 
-				octave_value_list xidx = idx.back ();
+                    retval.numel (get_numel (tmp, pidx));
 
-				octave_idx_type nidx = xidx.length ();
+                    tmpi = i-1;
+                    tmpidx.push_back (tidx);
+                  }
+                else
+                  {
+                    if (tmp.is_undefined () || autoconv)
+                      {
+                        tmpi = i+1;
+                        tmp = octave_value ();
+                      }
+                    else
+                      {
+                        retval.numel (get_numel (tmp, octave_value_list ()));
 
-				for (octave_idx_type j = 0; j < nidx; j++)
-				  {
-				    octave_value val = xidx(j);
+                        tmpi = i;
+                        tmpidx.push_back (tidx);
+                      }
+                  }
 
-				    nel *= val.numel ();
-				  }
+                if (error_state)
+                  break;
 
-				retval.numel (nel);
-			      }
-			    else if (first_retval_object.is_defined ())
-			      {
-				octave_value_list tmp_list
-				  = first_retval_object.subsref (ttype, idx, 1);
-
-				if (! error_state)
-				  {
-				    octave_value val = tmp_list(0);
-
-				    retval.numel (val.numel ());
-				  }
-			      }
-			    else
-			      retval.numel (1);
-			  }
-			else
-			  {
-			    if (first_retval_object.is_defined ())
-			      retval.numel (first_retval_object.numel ());
-			    else
-			      retval.numel (1);
-			  }
-		      }
-		    else
-		      {
-			octave_value tobj = first_retval_object;
-
-			if (! have_new_struct_field)
-			  {
-			    if (i > 0 && first_retval_object.is_defined ())
-			      {
-				std::string ttype = type.substr (0, i);
-
-				char c = ttype[ttype.length()-1];
-
-				if (! (c == '(' || c == '{'))
-				  {
-				    octave_value_list tmp_list
-				      = first_retval_object.subsref (ttype, idx, 1);
-
-				    if (! error_state)
-				      tobj = tmp_list(0);
-				  }
-			      }
-
-			    if (! error_state && tobj.is_map ())
-			      {
-				if (tidx.is_string ())
-				  {
-				    Octave_map m = tobj.map_value ();
-
-				    std::string s = tidx.string_value ();
-
-				    if (! m.contains (s))
-				      {
-					have_new_struct_field = true;
-
-					new_struct_field_nel = m.numel ();
-				      }
-				  }
-			      }
-			  }
-		      }
-
-		    idx.push_back (octave_value (tidx));
-		  }
-		else
-		  eval_error ();
+                idx.push_back (tidx);
 	      }
 	      break;
 
@@ -588,6 +577,9 @@ tree_index_expression::lvalue (void)
 	      panic_impossible ();
 	    }
 
+          if (idx.back ().empty ())
+            error ("invalid empty index list");
+
 	  if (error_state)
 	    break;
 
@@ -598,6 +590,7 @@ tree_index_expression::lvalue (void)
 
       if (! error_state)
 	retval.set_index (type, idx);
+
     }
 
   return retval;
@@ -628,44 +621,24 @@ tree_index_expression::lvalue (void)
 %! assert (x(2).b == 1);
 */
 
-void
-tree_index_expression::eval_error (void) const
-{
-  int l = line ();
-  int c = column ();
-
-  const char *type_str;
-
-  if (type[0] == '.')
-    type_str = "structure reference operator";
-  else if (args.front ())
-    type_str = "index expression";
-  else
-    type_str = "expression";
-
-  if (l != -1 && c != -1)
-    ::error ("evaluating %s near line %d, column %d", type_str, l, c);
-  else
-    ::error ("evaluating %s", type_str);
-}
-
 tree_index_expression *
-tree_index_expression::dup (symbol_table *sym_tab)
+tree_index_expression::dup (symbol_table::scope_id scope,
+			    symbol_table::context_id context) const
 {
   tree_index_expression *new_idx_expr
     = new tree_index_expression (line (), column ());
 
-  new_idx_expr->expr = expr ? expr->dup (sym_tab) : 0;
+  new_idx_expr->expr = expr ? expr->dup (scope, context) : 0;
 
   std::list<tree_argument_list *> new_args;
 
-  for (std::list<tree_argument_list *>::iterator p = args.begin ();
+  for (std::list<tree_argument_list *>::const_iterator p = args.begin ();
        p != args.end ();
        p++)
     {
-      tree_argument_list *elt = *p;
+      const tree_argument_list *elt = *p;
 
-      new_args.push_back (elt ? elt->dup (sym_tab) : 0);
+      new_args.push_back (elt ? elt->dup (scope, context) : 0);
     }
 
   new_idx_expr->args = new_args;
@@ -676,13 +649,13 @@ tree_index_expression::dup (symbol_table *sym_tab)
 
   std::list<tree_expression *> new_dyn_field;
 
-  for (std::list<tree_expression *>::iterator p = dyn_field.begin ();
+  for (std::list<tree_expression *>::const_iterator p = dyn_field.begin ();
        p != dyn_field.end ();
        p++)
     {
-      tree_expression *elt = *p;
+      const tree_expression *elt = *p;
 
-      new_dyn_field.push_back (elt ? elt->dup (sym_tab) : 0);
+      new_dyn_field.push_back (elt ? elt->dup (scope, context) : 0);
     }
 
   new_idx_expr->dyn_field = new_dyn_field;
