@@ -1,6 +1,7 @@
 /*
 
-Copyright (C) 1996, 1997, 2002, 2004, 2005, 2006, 2007 John W. Eaton
+Copyright (C) 1996, 1997, 2002, 2004, 2005, 2006, 2007, 2008, 2009
+              John W. Eaton
 
 This file is part of Octave.
 
@@ -26,6 +27,7 @@ along with Octave; see the file COPYING.  If not, see
 class tree_walker;
 
 #include "pt-cmd.h"
+#include "symtab.h"
 
 // Break.
 
@@ -39,9 +41,8 @@ public:
 
   ~tree_break_command (void) { }
 
-  void eval (void);
-
-  tree_command *dup (symbol_table *);
+  tree_command *dup (symbol_table::scope_id scope,
+		     symbol_table::context_id context) const;
 
   void accept (tree_walker& tw);
 
@@ -68,9 +69,8 @@ public:
 
   ~tree_continue_command (void) { }
 
-  void eval (void);
-
-  tree_command *dup (symbol_table *);
+  tree_command *dup (symbol_table::scope_id scope,
+		     symbol_table::context_id context) const;
 
   void accept (tree_walker& tw);
 
@@ -97,9 +97,8 @@ public:
 
   ~tree_return_command (void) { }
 
-  void eval (void);
-
-  tree_command *dup (symbol_table *);
+  tree_command *dup (symbol_table::scope_id scope,
+		     symbol_table::context_id context) const;
 
   void accept (tree_walker& tw);
 
