@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1994, 1995, 1996, 1997, 1999, 2000, 2002, 2003, 2004,
+              2005, 2006, 2007, 2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -25,6 +25,8 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include <config.h>
 #endif
 
+#include "Cell.h"
+
 #include "defun.h"
 #include "error.h"
 #include "ov.h"
@@ -35,10 +37,6 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "pt-walk.h"
 #include "utils.h"
 
-// If TRUE, print a warning if a function does not define all the
-// values in the return list which are expected.
-static bool Vwarn_undefined_return_values;
-
 // Parameter lists.
 
 tree_parameter_list::~tree_parameter_list (void)
@@ -56,11 +54,70 @@ tree_parameter_list::mark_as_formal_parameters (void)
 {
   for (iterator p = begin (); p != end (); p++)
     {
-      tree_identifier *elt = *p;
+      tree_decl_elt *elt = *p;
       elt->mark_as_formal_parameter ();
     }
 }
 
+bool
+tree_parameter_list::validate (in_or_out type)
+{
+  bool retval = true;
+
+  std::set<std::string> dict;
+
+  for (iterator p = begin (); p != end (); p++)
+    {
+      tree_decl_elt *elt = *p;
+
+      tree_identifier *id = elt->ident ();
+
+      if (id)
+	{
+	  std::string name = id->name ();
+
+	  if (dict.find (name) != dict.end ())
+	    {
+	      retval = false;
+	      error ("`%s' appears more than once in parameter list",
+		     name.c_str ());
+	      break;
+	    }
+	  else
+	    dict.insert (name);
+	}
+    }
+
+  if (! error_state)
+    {
+      std::string va_type = (type == in ? "varargin" : "varargout");
+
+      size_t len = length ();
+
+      if (len > 0)
+	{
+	  tree_decl_elt *elt = back ();
+
+	  tree_identifier *id = elt->ident ();
+
+	  if (id && id->name () == va_type)
+	    {
+	      if (len == 1)
+		mark_varargs_only ();
+	      else
+		mark_varargs ();
+
+	      iterator p = end ();
+	      --p;
+	      delete *p;
+	      erase (p);
+	    }
+	}
+    }
+
+  return retval;
+}
+
 void
 tree_parameter_list::initialize_undefined_elements (const std::string& warnfor,
 						    int nargout,
@@ -75,15 +132,18 @@ tree_parameter_list::initialize_undefined_elements (const std::string& warnfor,
       if (++count > nargout)
 	break;
 
-      tree_identifier *elt = *p;
+      tree_decl_elt *elt = *p;
 
-      if (! elt->is_defined ())
+      if (! elt->is_variable ())
 	{
-	  if (Vwarn_undefined_return_values && ! warned)
+	  if (! warned)
 	    {
 	      warned = true;
-	      warning ("%s: some elements in list of return values are undefined",
-		       warnfor.c_str ());
+
+	      warning_with_id
+		("Octave:undefined-return-values",
+		 "%s: some elements in list of return values are undefined",
+		 warnfor.c_str ());
 	    }
 
 	  octave_lvalue tmp = elt->lvalue ();
@@ -98,16 +158,13 @@ tree_parameter_list::define_from_arg_vector (const octave_value_list& args)
 {
   int nargin = args.length ();
 
-  if (nargin <= 0)
-    return;
-
   int expected_nargin = length ();
 
   iterator p = begin ();
 
   for (int i = 0; i < expected_nargin; i++)
     {
-      tree_identifier *elt = *p++;
+      tree_decl_elt *elt = *p++;
 
       octave_lvalue ref = elt->lvalue ();
 
@@ -115,14 +172,17 @@ tree_parameter_list::define_from_arg_vector (const octave_value_list& args)
 	{
 	  if (args(i).is_defined () && args(i).is_magic_colon ())
 	    {
-	      ::error ("invalid use of colon in function argument list");
-	      return;
+	      if (! elt->eval ())
+		{
+		  ::error ("no default value for argument %d\n", i+1);
+		  return;
+		}
 	    }
-
-	  ref.assign (octave_value::op_asn_eq, args(i));
+	  else
+	    ref.define (args(i));
 	}
       else
-	ref.assign (octave_value::op_asn_eq, octave_value ());
+	elt->eval ();
     }
 }
 
@@ -135,7 +195,7 @@ tree_parameter_list::undefine (void)
 
   for (int i = 0; i < len; i++)
     {
-      tree_identifier *elt = *p++;
+      tree_decl_elt *elt = *p++;
 
       octave_lvalue ref = elt->lvalue ();
 
@@ -144,34 +204,25 @@ tree_parameter_list::undefine (void)
 }
 
 octave_value_list
-tree_parameter_list::convert_to_const_vector (tree_va_return_list *vr_list)
+tree_parameter_list::convert_to_const_vector (const Cell& varargout)
 {
-  int nout = length ();
+  octave_idx_type vlen = varargout.numel ();
 
-  if (vr_list)
-    nout += vr_list->length ();
+  int nout = length () + vlen;
 
-  octave_value_list retval;
-  retval.resize (nout);
+  octave_value_list retval (nout, octave_value ());
 
   int i = 0;
 
   for (iterator p = begin (); p != end (); p++)
     {
-      tree_identifier *elt = *p;
+      tree_decl_elt *elt = *p;
 
-      retval(i++) = elt->is_defined () ? elt->rvalue () : octave_value ();
+      retval(i++) = elt->is_defined () ? elt->rvalue1 () : octave_value ();
     }
 
-  if (vr_list)
-    {
-      for (tree_va_return_list::iterator p = vr_list->begin ();
-	   p != vr_list->end ();
-	   p++)
-	{
-	  retval(i++) = *p;
-	}
-    }
+  for (octave_idx_type j = 0; j < vlen; j++)
+    retval(i++) = varargout(j);
 
   return retval;
 }
@@ -183,9 +234,9 @@ tree_parameter_list::is_defined (void)
 
   for (iterator p = begin (); p != end (); p++)
     {
-      tree_identifier *elt = *p;
+      tree_decl_elt *elt = *p;
 
-      if (! elt->is_defined ())
+      if (! elt->is_variable ())
 	{
 	  status = false;
 	  break;
@@ -195,6 +246,25 @@ tree_parameter_list::is_defined (void)
   return status;
 }
 
+tree_parameter_list *
+tree_parameter_list::dup (symbol_table::scope_id scope,
+			  symbol_table::context_id context) const
+{
+  tree_parameter_list *new_list = new tree_parameter_list ();
+
+  if (takes_varargs ())
+    new_list->mark_varargs ();
+
+  for (const_iterator p = begin (); p != end (); p++)
+    {
+      const tree_decl_elt *elt = *p;
+
+      new_list->append (elt->dup (scope, context));
+    }
+
+  return new_list;
+}
+
 void
 tree_parameter_list::accept (tree_walker& tw)
 {
@@ -213,33 +283,28 @@ tree_return_list::~tree_return_list (void)
     }
 }
 
+tree_return_list *
+tree_return_list::dup (symbol_table::scope_id scope,
+		       symbol_table::context_id context) const
+{
+  tree_return_list *new_list = new tree_return_list ();
+
+  for (const_iterator p = begin (); p != end (); p++)
+    {
+      const tree_index_expression *elt = *p;
+
+      new_list->append (elt->dup (scope, context));
+    }
+
+  return new_list;
+}
+
 void
 tree_return_list::accept (tree_walker& tw)
 {
   tw.visit_return_list (*this);
 }
 
-static int
-warn_undefined_return_values (void)
-{
-  Vwarn_undefined_return_values
-    = check_preference ("warn_undefined_return_values");
-
-  return 0;
-}
-
-void
-symbols_of_pt_misc (void)
-{
-  DEFVAR (warn_undefined_return_values, true, warn_undefined_return_values,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} warn_undefined_return_values\n\
-If the value of @code{warn_undefined_return_values} is nonzero,\n\
-print a warning if a function does not define all the values in\n\
-the return list which are expected.  The default value is 1.\n\
-@end defvr");
-}
-
 /*
 ;;; Local Variables: ***
 ;;; mode: C++ ***
