@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2002,
+              2003, 2004, 2005, 2006, 2007, 2008, 2009 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -38,19 +38,23 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #endif
 
 #include "cmd-edit.h"
+#include "oct-syscalls.h"
 #include "quit.h"
 
+#include "debug.h"
 #include "defun.h"
 #include "error.h"
 #include "load-save.h"
 #include "oct-map.h"
 #include "pager.h"
 #include "pt-bp.h"
+#include "pt-eval.h"
 #include "sighandlers.h"
 #include "sysdep.h"
 #include "syswait.h"
 #include "toplev.h"
 #include "utils.h"
+#include "variables.h"
 
 // Nonzero means we have already printed a message for this series of
 // SIGPIPES.  We assume that the writer will eventually give up.
@@ -65,12 +69,12 @@ static bool Vdebug_on_interrupt = false;
 // Allow users to avoid writing octave-core for SIGHUP (sent by
 // closing gnome-terminal, for example).  Note that this variable has
 // no effect if Vcrash_dumps_octave_core is FALSE.
-static bool Vsighup_dumps_octave_core;
+static bool Vsighup_dumps_octave_core = true;
 
 // Similar to Vsighup_dumps_octave_core, but for SIGTERM signal.
-static bool Vsigterm_dumps_octave_core;
+static bool Vsigterm_dumps_octave_core = true;
 
-#if RETSIGTYPE == void
+#if defined (RETSIGTYPE_IS_VOID)
 #define SIGHANDLER_RETURN(status) return
 #else
 #define SIGHANDLER_RETURN(status) return status
@@ -114,17 +118,21 @@ octave_signal_handler (void)
 
 	  switch (i)
 	    {
+#ifdef SIGCHLD
 	    case SIGCHLD:
 	      octave_child_list::reap ();
 	      break;
+#endif
 
 	    case SIGFPE:
 	      std::cerr << "warning: floating point exception -- trying to return to prompt" << std::endl;
 	      break;
 
+#ifdef SIGPIPE
 	    case SIGPIPE:
 	      std::cerr << "warning: broken pipe -- some output may be lost" << std::endl;
 	      break;
+#endif
 	    }
 	}
     }
@@ -143,6 +151,9 @@ my_friendly_exit (const char *sig_name, int sig_number,
 #endif
 
       std::cerr << "panic: attempted clean up apparently failed -- aborting...\n";
+
+      MINGW_SIGNAL_CLEANUP ();
+
       abort ();
     }
   else
@@ -155,7 +166,11 @@ my_friendly_exit (const char *sig_name, int sig_number,
 	dump_octave_core ();
 
       if (sig_number < 0)
-	exit (1);
+	{
+	  MINGW_SIGNAL_CLEANUP ();
+
+	  exit (1);
+	}
       else
 	{
 	  octave_set_signal_handler (sig_number, SIG_DFL);
@@ -173,7 +188,8 @@ my_friendly_exit (const char *sig_name, int sig_number,
 }
 
 sig_handler *
-octave_set_signal_handler (int sig, sig_handler *handler)
+octave_set_signal_handler (int sig, sig_handler *handler,
+			   bool restart_syscalls)
 {
 #if defined (HAVE_POSIX_SIGNALS)
   struct sigaction act, oact;
@@ -187,12 +203,11 @@ octave_set_signal_handler (int sig, sig_handler *handler)
       act.sa_flags |= SA_INTERRUPT;
 #endif
     }
-  else
-    {
 #if defined (SA_RESTART)
-      act.sa_flags |= SA_RESTART;
+  // FIXME -- Do we also need to explicitly disable SA_RESTART?
+  else if (restart_syscalls)
+    act.sa_flags |= SA_RESTART;
 #endif
-    }
 
   sigemptyset (&act.sa_mask);
   sigemptyset (&oact.sa_mask);
@@ -208,7 +223,7 @@ octave_set_signal_handler (int sig, sig_handler *handler)
 static RETSIGTYPE
 generic_sig_handler (int sig)
 {
-  my_friendly_exit (sys_siglist[sig], sig);
+  my_friendly_exit (strsignal (sig), sig);
 
   SIGHANDLER_RETURN (0);
 }
@@ -343,13 +358,9 @@ sigwinch_handler (int /* sig */)
 // use the value of sig, instead of just assuming that it is called
 // for SIGINT only.
 
-static RETSIGTYPE
-sigint_handler (int sig)
+static void
+user_abort (const char *sig_name, int sig_number)
 {
-  MAYBE_ACK_SIGNAL (sig);
-
-  MAYBE_REINSTALL_SIGHANDLER (sig, sigint_handler);
-
   if (! octave_initialized)
     exit (1);
 
@@ -359,17 +370,27 @@ sigint_handler (int sig)
 	{
 	  if (! octave_debug_on_interrupt_state)
 	    {
+	      tree_evaluator::debug_mode = true;
 	      octave_debug_on_interrupt_state = true;
 
-	      SIGHANDLER_RETURN (0);
+	      return;
 	    }
 	  else
-	    // Clear the flag and do normal interrupt stuff.
-	    octave_debug_on_interrupt_state = false;
+	    {
+	      // Clear the flag and do normal interrupt stuff.
+
+	      tree_evaluator::debug_mode = bp_table::have_breakpoints ();
+	      octave_debug_on_interrupt_state = false;
+	    }
 	}
 
       if (octave_interrupt_immediately)
-	octave_jump_to_enclosing_context ();
+	{
+	  if (octave_interrupt_state == 0)
+	    octave_interrupt_state = 1;
+
+	  octave_jump_to_enclosing_context ();
+	}
       else
 	{
 	  // If we are already cleaning up from a previous interrupt,
@@ -386,10 +407,28 @@ sigint_handler (int sig)
 	    std::cerr << "Press Control-C again to abort." << std::endl;
 
 	  if (octave_interrupt_state >= 3)
-	    my_friendly_exit (sys_siglist[sig], sig, true);
+	    my_friendly_exit (sig_name, sig_number, true);
 	}
     }
 
+}
+
+static RETSIGTYPE
+sigint_handler (int sig)
+{
+  MAYBE_ACK_SIGNAL (sig);
+
+  MAYBE_REINSTALL_SIGHANDLER (sig, sigint_handler);
+
+#ifdef USE_W32_SIGINT
+  if (w32_in_main_thread ())
+    user_abort (strsignal (sig), sig);
+  else
+    w32_raise (sig);
+#else
+  user_abort (strsignal (sig), sig);
+#endif
+
   SIGHANDLER_RETURN (0);
 }
 
@@ -414,6 +453,60 @@ sigpipe_handler (int /* sig */)
 }
 #endif /* defined(SIGPIPE) */
 
+#ifdef USE_W32_SIGINT
+static BOOL CALLBACK
+w32_sigint_handler (DWORD sig)
+{
+  const char *sig_name;
+
+  switch(sig)
+    {
+      case CTRL_BREAK_EVENT:   
+	sig_name = "Ctrl-Break"; 
+	break;
+      case CTRL_C_EVENT:
+	sig_name = "Ctrl-C";
+	break;
+      case CTRL_CLOSE_EVENT:
+	sig_name = "close console";
+	break;
+      case CTRL_LOGOFF_EVENT:
+	sig_name = "logoff";
+	break;
+      case CTRL_SHUTDOWN_EVENT:
+	sig_name = "shutdown";
+	break;
+      default:
+	sig_name = "unknown console event";
+	break;
+    }
+
+  switch(sig)
+    {
+      case CTRL_BREAK_EVENT:
+      case CTRL_C_EVENT:
+	w32_raise (SIGINT);
+        break;
+
+      case CTRL_CLOSE_EVENT:
+      case CTRL_LOGOFF_EVENT:
+      case CTRL_SHUTDOWN_EVENT:
+      default:
+        // We should do the following:
+        //    clean_up_and_exit (0);
+        // We can't because we aren't running in the normal Octave thread.
+	user_abort(sig_name, sig);
+        break;
+    }
+
+  // Return TRUE if the event was handled, or FALSE if another handler 
+  // should be called.
+  // FIXME check that windows terminates the thread.
+  return TRUE;
+}
+#endif /* w32_sigint_handler */
+
+
 octave_interrupt_handler
 octave_catch_interrupts (void)
 {
@@ -427,6 +520,22 @@ octave_catch_interrupts (void)
   retval.brk_handler = octave_set_signal_handler (SIGBREAK, sigint_handler);
 #endif
 
+#ifdef USE_W32_SIGINT
+
+  // Intercept windows console control events.
+  // Note that the windows console signal handlers chain, so if 
+  // install_signal_handlers is called more than once in the same program,
+  // then first call the following to avoid duplicates:
+  //
+  //   SetConsoleCtrlHandler (w32_sigint_handler, FALSE);
+
+  if (! SetConsoleCtrlHandler (w32_sigint_handler, TRUE))
+    error ("SetConsoleCtrlHandler failed with %ld\n", GetLastError ());
+
+  w32_set_quiet_shutdown ();
+
+#endif
+
   return retval;
 }
 
@@ -447,16 +556,19 @@ octave_ignore_interrupts (void)
 }
 
 octave_interrupt_handler
-octave_set_interrupt_handler (const volatile octave_interrupt_handler& h)
+octave_set_interrupt_handler (const volatile octave_interrupt_handler& h,
+			      bool restart_syscalls)
 {
   octave_interrupt_handler retval;
 
 #ifdef SIGINT
-  retval.int_handler = octave_set_signal_handler (SIGINT, h.int_handler);
+  retval.int_handler = octave_set_signal_handler (SIGINT, h.int_handler,
+						  restart_syscalls);
 #endif
 
 #ifdef SIGBREAK
-  retval.brk_handler = octave_set_signal_handler (SIGBREAK, h.brk_handler);
+  retval.brk_handler = octave_set_signal_handler (SIGBREAK, h.brk_handler,
+						  restart_syscalls);
 #endif
 
   return retval;
@@ -530,10 +642,7 @@ install_signal_handlers (void)
   octave_set_signal_handler (SIGPOLL, SIG_IGN);
 #endif
 
-#ifdef SIGPROF
-  octave_set_signal_handler (SIGPROF, generic_sig_handler);
-#endif
-
+  // SIGPROF
   // SIGPWR
 
 #ifdef SIGQUIT
@@ -592,6 +701,7 @@ install_signal_handlers (void)
 #ifdef SIGXFSZ
   octave_set_signal_handler (SIGXFSZ, generic_sig_handler);
 #endif
+
 }
 
 static Octave_map
@@ -855,7 +965,7 @@ OCL_REP::wait (void)
 	{
 	  int status;
 
-	  if (waitpid (pid, &status, WNOHANG) > 0)
+	  if (octave_syscalls::waitpid (pid, &status, WNOHANG) > 0)
 	    {
 	      oc.have_status = 1;
 
@@ -871,66 +981,61 @@ OCL_REP::wait (void)
   return retval;
 }
 
-static int
-debug_on_interrupt (void)
+DEFUN (SIG, args, ,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {} SIG ()\n\
+Return a structure containing Unix signal names and their defined values.\n\
+@end deftypefn")
 {
-  Vdebug_on_interrupt = check_preference ("debug_on_interrupt");
+  octave_value retval;
 
-  return 0;
+  if (args.length () == 0)
+    {
+      static Octave_map m = make_sig_struct ();
+
+      retval = m;
+    }
+  else
+    print_usage ();
+
+  return retval;
 }
 
-static int
-sighup_dumps_octave_core (void)
-{
-  Vsighup_dumps_octave_core = check_preference ("sighup_dumps_octave_core");
-
-  return 0;
-}
-
-static int
-sigterm_dumps_octave_core (void)
-{
-  Vsigterm_dumps_octave_core = check_preference ("sigterm_dumps_octave_core");
-
-  return 0;
-}
-
-void
-symbols_of_sighandlers (void)
-{
-  DEFVAR (debug_on_interrupt, false, debug_on_interrupt,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} debug_on_interrupt\n\
-If @code{debug_on_interrupt} is nonzero, Octave will try to enter\n\
-debugging mode when it receives an interrupt signal (typically\n\
+DEFUN (debug_on_interrupt, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} debug_on_interrupt ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} debug_on_interrupt (@var{new_val})\n\
+Query or set the internal variable that controls whether Octave will try\n\
+to enter debugging mode when it receives an interrupt signal (typically\n\
 generated with @kbd{C-c}).  If a second interrupt signal is received\n\
 before reaching the debugging mode, a normal interrupt will occur.\n\
-The default value is 0.\n\
-@end defvr");
+@end deftypefn")
+{
+  return SET_INTERNAL_VARIABLE (debug_on_interrupt);
+}
 
-  DEFVAR (sighup_dumps_octave_core, true, sighup_dumps_octave_core,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} sighup_dumps_octave_core\n\
-If this variable is set to a nonzero value and\n\
-@code{crash_dumps_octave_core} is also nonzero, Octave tries to save all\n\
-current variables the the file \"octave-core\" if it receives a\n\
-hangup signal.  The default value is 1.\n\
-@end defvr");
+DEFUN (sighup_dumps_octave_core, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} sighup_dumps_octave_core ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} sighup_dumps_octave_core (@var{new_val})\n\
+Query or set the internal variable that controls whether Octave tries\n\
+to save all current variables to the file \"octave-core\" if it receives\n\
+a hangup signal.\n\
+@end deftypefn")
+{
+  return SET_INTERNAL_VARIABLE (sighup_dumps_octave_core);
+}
 
-  DEFVAR (sigterm_dumps_octave_core, true, sigterm_dumps_octave_core,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} sigterm_dumps_octave_core\n\
-If this variable is set to a nonzero value and\n\
-@code{crash_dumps_octave_core} is also nonzero, Octave tries to save all\n\
-current variables the the file \"octave-core\" if it receives a\n\
-terminate signal.  The default value is 1.\n\
-@end defvr");
-
-  DEFCONST (SIG, make_sig_struct (),
-    "-*- texinfo -*-\n\
-@defvr {Built-in Constant} SIG\n\
-Structure of Unix signal names and their defined values.\n\
-@end defvr");
+DEFUN (sigterm_dumps_octave_core, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} sigterm_dumps_octave_core ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} sigterm_dumps_octave_core (@var{new_val})\n\
+Query or set the internal variable that controls whether Octave tries\n\
+to save all current variables to the file \"octave-core\" if it receives\n\
+a terminate signal.\n\
+@end deftypefn")
+{
+  return SET_INTERNAL_VARIABLE (sigterm_dumps_octave_core);
 }
 
 /*
