@@ -939,11 +939,12 @@ public:
 		instance = p->second;
 
 		xcurrent_scope = scope;
+
+                xcurrent_context = context;
 	      }
 	  }
-
-	if (! error_state)
-	  xcurrent_context = context;
+        else
+          xcurrent_context = context;
       }
   }
 
@@ -1062,8 +1063,6 @@ public:
 			      scope_id scope = xcurrent_scope,
 			      context_id context = xcurrent_context)
   {
-    assert (xcurrent_context == 0);
-
     symbol_table *inst = get_instance (scope);
 
     if (inst)
@@ -2143,15 +2142,14 @@ private:
 	symbol_record& sr = p->second;
 
 	if (sr.is_global ())
-	  {
-	    global_table_iterator q = global_table.find (name);
-
-	    if (q != global_table.end ())
-	      global_table.erase (q);
-
-	    sr.unmark_global ();
-	  }
+          sr.unmark_global ();
       }
+
+    global_table_iterator q = global_table.find (name);
+
+    if (q != global_table.end ())
+      global_table.erase (q);
+
   }
 
   void do_clear_variable (const std::string& name)
@@ -2170,19 +2168,18 @@ private:
       {
 	symbol_record& sr = p->second;
 
-	if (sr.is_global ())
-	  {
-	    if (pattern.match (sr.name ()))
-	      {
-		global_table_iterator q = global_table.find (sr.name ());
-
-		if (q != global_table.end ())
-		  global_table.erase (q);
-
-		sr.unmark_global ();
-	      }
-	  }
+	if (sr.is_global () && pattern.match (sr.name ()))
+          sr.unmark_global ();
       }
+
+    for (global_table_iterator q = global_table.begin (); 
+         q != global_table.end (); q++)
+      {
+	if (pattern.match (q->first))
+          global_table.erase (q);
+      }
+
+
   }
 
   void do_clear_variable_pattern (const std::string& pat)
