@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2002,
-              2003, 2005, 2006, 2007 John W. Eaton
+              2003, 2005, 2006, 2007, 2008 John W. Eaton
 
 This file is part of Octave.
 
@@ -26,7 +26,7 @@ along with Octave; see the file COPYING.  If not, see
 
 #include <cstdarg>
 
-#include <iostream>
+#include <iosfwd>
 #include <string>
 
 #include "dMatrix.h"
@@ -64,6 +64,8 @@ search_path_for_all_files (const std::string&, const string_vector&);
 extern OCTINTERP_API std::string
 file_in_path (const std::string&, const std::string&);
 
+extern OCTINTERP_API std::string contents_file_in_path (const std::string&);
+
 extern OCTINTERP_API std::string fcn_file_in_path (const std::string&);
 extern OCTINTERP_API std::string oct_file_in_path (const std::string&);
 extern OCTINTERP_API std::string mex_file_in_path (const std::string&);
@@ -93,6 +95,9 @@ get_dimensions (const octave_value& a,const char *warn_for,
 extern OCTINTERP_API Matrix
 identity_matrix (octave_idx_type nr, octave_idx_type nc);
 
+extern OCTINTERP_API FloatMatrix
+float_identity_matrix (octave_idx_type nr, octave_idx_type nc);
+
 extern OCTINTERP_API int
 octave_format (std::ostream& os, const char *fmt, ...);
 
